#############################################################################################
#                                                                                           #
# Author:   GeoPy Team                                                                      #
# Email:    geopy.info@gmail.com                                                            #
# Date:     January 2019                                                                    #
#                                                                                           #
#############################################################################################

# basic functions for lines

import sys, os
#
sys.path.append(os.path.dirname(__file__)[:-4][:-4][:-13])
from cognitivegeo.src.vis.color import color as color


__all__ = ['line']

# List of all available line properties, including Style and Width
LineStyleList = ['Solid', 'Dashed', 'Dashdot', 'Dotted', 'None']
LineWidthList = [i for i in range(1, 20)]


class line:
    # Pack all functions as a class
    #
    LineStyleList = LineStyleList
    LineWidthList = LineWidthList
    LineColorList = color.ColorList


#############################################################################################
#                                                                                           #
# Author:   GeoPy Team                                                                      #
# Email:    geopy.info@gmail.com                                                            #
# Date:     December 2019                                                                   #
#                                                                                           #
#############################################################################################

# basic functions for messager

import sys
import os
#
sys.path.append(os.path.dirname(__file__)[:-4][:-4][:-13])

# Print format
# Text color: black (30), red (31), green (32), yellow (33), blue (34), purple (35), cyan (36), white (37)
# Text style: no effect (0), bold (1), underline (2), negative1 (3), negative2 (5)
# Background color: black (40), red (41), green (42), yellow (43), blue (44), purple (45), cyan (46),white (47)


__all__ = ['messager']

def print2Terminal(message, type='Normal'):
    """
    Print message to terminal with format, to differentiate errors and warnings

    Args:
        message:    A string for print
        type:       Message type: 'Normal', 'Warning', 'Error'

    Return:
         N/A
    """
    if type != 'Normal' and type != 'normal' and type != 'NORMAL' \
        and type != 'Warning' and type != 'warning' and type != 'WARNING' \
        and type != 'Error' and type != 'error' and type != 'ERROR':
        type = 'normal'
    if type == 'normal' or type == 'Normal' or type == 'NORMAL':
        # print('\033[0;30;47m' + message)
        print('\033[0;0;0m' + message)
    if type == 'Warning' or type == 'warning' or type == 'WARNING':
        print('\033[1;34;40m' + message)
    if type == 'Error' or type == 'error' or type == 'ERROR':
        print('\033[1;31;40m' + message)
    # reset
    print('\033[0;0;0m', end='\r')

class messager:
    print = print2Terminal


#############################################################################################
#                                                                                           #
# Author:   GeoPy Team                                                                      #
# Email:    geopy.info@gmail.com                                                            #
# Date:     January 2019                                                                    #
#                                                                                           #
#############################################################################################

# Create a window for converting pre-stack to seismic


from PyQt5 import QtCore, QtGui, QtWidgets
import numpy as np
import os, sys
#
sys.path.append(os.path.dirname(__file__)[:-4][:-4][:-13])
from cognitivegeo.src.psseismic.analysis import analysis as psseis_ays
from cognitivegeo.src.seismic.analysis import analysis as seis_ays
from cognitivegeo.src.vis.messager import messager as vis_msg

QtWidgets.QApplication.setAttribute(QtCore.Qt.AA_EnableHighDpiScaling, True)
QtWidgets.QApplication.setAttribute(QtCore.Qt.AA_UseHighDpiPixmaps, True)


class convertpsseis2seis(object):

    survinfo = {}
    seisdata = {}
    psseisdata = {}
    #
    iconpath = os.path.dirname(__file__)
    dialog = None


    def setupGUI(self, ConvertPsSeis2Seis):
        ConvertPsSeis2Seis.setObjectName("ConvertPsSeis2Seis")
        ConvertPsSeis2Seis.setFixedSize(400, 410)
        icon = QtGui.QIcon()
        icon.addPixmap(QtGui.QPixmap(os.path.join(self.iconpath, "icons/psseismic.png")),
                       QtGui.QIcon.Normal,
                       QtGui.QIcon.Off)
        ConvertPsSeis2Seis.setWindowIcon(icon)
        #
        self.lblpsseis = QtWidgets.QLabel(ConvertPsSeis2Seis)
        self.lblpsseis.setObjectName("lblpsseis")
        self.lblpsseis.setGeometry(QtCore.QRect(10, 10, 170, 30))
        self.lwgpsseis = QtWidgets.QListWidget(ConvertPsSeis2Seis)
        self.lwgpsseis.setObjectName("lwgpsseis")
        self.lwgpsseis.setGeometry(QtCore.QRect(10, 50, 170, 200))
        self.lwgpsseis.setSelectionMode(QtWidgets.QAbstractItemView.SingleSelection)
        self.lblarrow = QtWidgets.QLabel(ConvertPsSeis2Seis)
        self.lblarrow.setObjectName("lblarrow")
        self.lblarrow.setGeometry(QtCore.QRect(180, 110, 40, 30))
        self.lblpsshot = QtWidgets.QLabel(ConvertPsSeis2Seis)
        self.lblpsshot.setObjectName("lblpsshot")
        self.lblpsshot.setGeometry(QtCore.QRect(220, 10, 170, 30))
        self.lwgpsshot = QtWidgets.QListWidget(ConvertPsSeis2Seis)
        self.lwgpsshot.setObjectName("lwgshot")
        self.lwgpsshot.setGeometry(QtCore.QRect(220, 50, 170, 200))
        self.lwgpsshot.setSelectionMode(QtWidgets.QAbstractItemView.SingleSelection)
        self.btnapply = QtWidgets.QPushButton(ConvertPsSeis2Seis)
        self.btnapply.setObjectName("btnedit")
        self.btnapply.setGeometry(QtCore.QRect(150, 360, 100, 30))
        icon = QtGui.QIcon()
        icon.addPixmap(QtGui.QPixmap(os.path.join(self.iconpath, "icons/ok.png")),
                       QtGui.QIcon.Normal,
                       QtGui.QIcon.Off)
        self.btnapply.setIcon(icon)
        #
        self.msgbox = QtWidgets.QMessageBox(ConvertPsSeis2Seis)
        self.msgbox.setObjectName("msgbox")
        _center_x = ConvertPsSeis2Seis.geometry().center().x()
        _center_y = ConvertPsSeis2Seis.geometry().center().y()
        self.msgbox.setGeometry(QtCore.QRect(_center_x - 150, _center_y - 50, 300, 100))
        #
        self.retranslateGUI(ConvertPsSeis2Seis)
        QtCore.QMetaObject.connectSlotsByName(ConvertPsSeis2Seis)


    def retranslateGUI(self, ConvertPsSeis2Seis):
        self.dialog = ConvertPsSeis2Seis
        #
        _translate = QtCore.QCoreApplication.translate
        ConvertPsSeis2Seis.setWindowTitle(_translate("ConvertPsSeis2Seis", "Convert Pre-stack to Seismic"))
        self.lblpsseis.setText(_translate("ConvertPsSeis2Seis", "Select pre-stack:"))
        self.lwgpsseis.itemSelectionChanged.connect(self.changeLwgPsSeis)
        self.lblarrow.setText(_translate("ConvertPsSeis2Seis", "==>"))
        self.lblarrow.setAlignment(QtCore.Qt.AlignCenter)
        self.lblpsshot.setText(_translate("ConvertPsSeis2Seis", "Select shot:"))
        self.btnapply.setText(_translate("ConvertPsSeis2Seis", "Apply"))
        self.btnapply.clicked.connect(self.clickBtnApply)
        #
        self.lwgpsseis.clear()
        if len(self.psseisdata.keys()) > 0:
            for i in sorted(self.psseisdata.keys()):
                if self.checkPsSeisData(i):
                    item = QtWidgets.QListWidgetItem(self.lwgpsseis)
                    item.setText(i)
                    self.lwgpsseis.addItem(item)


    def changeLwgPsSeis(self):
        self.lwgpsshot.clear()
        _firstshot = None
        #
        _psseislist = self.lwgpsseis.selectedItems()
        _psseislist = [f.text() for f in _psseislist]
        if len(_psseislist) > 0:
            for _k in self.psseisdata[_psseislist[0]].keys():
                item = QtWidgets.QListWidgetItem(self.lwgpsshot)
                item.setText(_k)
                self.lwgpsshot.addItem(item)
                if _firstshot is None:
                    _firstshot = item
        if _firstshot is not None:
            self.lwgpsshot.setCurrentItem(_firstshot)


    def clickBtnApply(self):
        self.refreshMsgBox()
        #
        _psseislist = self.lwgpsseis.selectedItems()
        _psseislist = [f.text() for f in _psseislist]
        if len(_psseislist) < 1:
            vis_msg.print("ERROR in ConvertPsSeis2Seis: No pre-stack selected for conversion", type='error')
            QtWidgets.QMessageBox.critical(self.msgbox,
                                           'Convert Pre-stack to Seismic',
                                           'No pre-stack selected for conversion')
            return
        #
        _psshotlist = self.lwgpsshot.selectedItems()
        _psshotlist = [f.text() for f in _psshotlist]
        if len(_psshotlist) < 1:
            vis_msg.print("ERROR in ConvertPsSeis2Seis: No shot selected for conversion", type='error')
            QtWidgets.QMessageBox.critical(self.msgbox,
                                            'Convert Pre-stack to Seismic',
                                            'No shot selected for conversion')
            return
        #
        _psseisdata = self.psseisdata[_psseislist[0]][_psshotlist[0]]['ShotData']
        _psseisinfo = self.psseisdata[_psseislist[0]][_psshotlist[0]]['ShotInfo']
        _nz, _nt, _nl = np.shape(_psseisdata)
        if checkSurvInfo(self.survinfo) \
                and self.survinfo['ZNum'] == np.shape(_psseisdata)[0] \
                and self.survinfo['XLNum'] == np.shape(_psseisdata)[1] \
                and self.survinfo['ILNum'] == np.shape(_psseisdata)[2]:
            _survinfo = self.survinfo
        else:
            _survinfo = seis_ays.createSeisInfoFrom3DMat(_psseisdata,
                                                         zstart=_psseisinfo['ZStart'],
                                                         zstep=_psseisinfo['ZStep'],
                                                         xlstart=_psseisinfo['XLStart'],
                                                         xlstep=_psseisinfo['XLStep'],
                                                         inlstart=_psseisinfo['ILStart'],
                                                         inlstep=_psseisinfo['ILStep']
                                                         )
        #
        _seisdata = {}
        _seisdata[_psshotlist[0]] = _psseisdata
        #
        # add new data to seisdata
        if checkSurvInfo(_survinfo):
            self.survinfo = _survinfo
        for key in _seisdata.keys():
            if key in self.seisdata.keys() and checkSeisData(self.seisdata[key], self.survinfo):
                reply = QtWidgets.QMessageBox.question(self.msgbox, 'Convert Pre-stack to Seismic',
                                                       key + ' already exists. Overwrite?',
                                                       QtWidgets.QMessageBox.Yes | QtWidgets.QMessageBox.No,
                                                       QtWidgets.QMessageBox.No)
                if reply == QtWidgets.QMessageBox.No:
                    return
            self.seisdata[key] = _seisdata[key]
        #
        QtWidgets.QMessageBox.information(self.msgbox,
                                          "Convert Pre-stack to Seismic",
                                          "Pre-stack seismic converted successfully")
        return


    def refreshMsgBox(self):
        _center_x = self.dialog.geometry().center().x()
        _center_y = self.dialog.geometry().center().y()
        self.msgbox.setGeometry(QtCore.QRect(_center_x - 150, _center_y - 50, 300, 100))


    def checkPsSeisData(self, name):
        return psseis_ays.checkPsSeis(self.psseisdata[name])


def checkSurvInfo(survinfo):
    return seis_ays.checkSeisInfo(survinfo)

def checkSeisData(seisdata, survinfo):
    return seis_ays.isSeis3DMatConsistentWithSeisInfo(seisdata, survinfo)


if __name__ == "__main__":
    import sys
    app = QtWidgets.QApplication(sys.argv)
    ConvertPsSeis2Seis = QtWidgets.QWidget()
    gui = convertpsseis2seis()
    gui.setupGUI(ConvertPsSeis2Seis)
    ConvertPsSeis2Seis.show()
    sys.exit(app.exec_())


from __future__ import print_function

from array import array
from collections import OrderedDict
from itertools import zip_longest, count

import os
import struct
import re
import logging
import sys
sys.path.append(os.path.dirname(__file__)[:-6][:-4][:-13])
import cognitivegeo.src.segpy.textual_reel_header
from cognitivegeo.src.segpy.binary_reel_header import BinaryReelHeader
from cognitivegeo.src.segpy.catalog import CatalogBuilder
from cognitivegeo.src.segpy.datatypes import SEG_Y_TYPE_TO_CTYPE, size_in_bytes, DATA_SAMPLE_FORMAT_TO_SEG_Y_TYPE, CTYPE_TO_SIZE
from cognitivegeo.src.segpy.encoding import guess_encoding, is_supported_encoding, UnsupportedEncodingError
from cognitivegeo.src.segpy.ibm_float import IBMFloat
from cognitivegeo.src.segpy.packer import HeaderPacker
from cognitivegeo.src.segpy.revisions import canonicalize_revision
from cognitivegeo.src.segpy.trace_header import TraceHeaderRev1
from cognitivegeo.src.segpy.util import file_length, batched, pad, complementary_intervals, NATIVE_ENDIANNESS, EMPTY_BYTE_STRING


HEADER_NEWLINE = '\r\n'

CARD_LENGTH = 80
CARDS_PER_HEADER = 40

TEXTUAL_HEADER_NUM_BYTES = CARD_LENGTH * CARDS_PER_HEADER
BINARY_HEADER_NUM_BYTES = 400
REEL_HEADER_NUM_BYTES = TEXTUAL_HEADER_NUM_BYTES + BINARY_HEADER_NUM_BYTES
TRACE_HEADER_NUM_BYTES = 240

END_TEXT_STANZA = "((SEG: EndText))"


def logger():
    # Defer logger creation until the module is *used* rather than imported.
    return logging.getLogger(__name__)


def extract_revision(binary_reel_header):
    """Obtain the SEG Y revision from the reel header.

    Args:
        binary_reel_header: A dictionary containing a reel header, such as obtained
            from read_binary_reel_header()

    Returns:
        One of the constants revisions.SEGY_REVISION_0 or
        revisions.SEGY_REVISION_1
    """
    raw_revision = binary_reel_header.format_revision_num
    return canonicalize_revision(raw_revision)


def num_extended_textual_headers(binary_reel_header):
    """Obtain the number of 3200 byte extended textual file headers.

    A value of zero indicates there are no Extended Textual File Header records
    (i.e. this file has no Extended Textual File Header(s)). A value of -1 indicates
    that there are a variable number of Extended Textual File Header records and the
    end of the Extended Textual File Header is denoted by an ((SEG: EndText)) stanza
    in the final record. A positive value indicates that there are exactly that many
    Extended Textual File Header records.
    """
    # TODO: Is this method needed any more?
    num_ext_headers = binary_reel_header.num_extended_textual_headers
    return num_ext_headers


def bytes_per_sample(binary_reel_header, revision):
    """Determine the number of bytes per sample from the reel header.

    Args:
        binary_reel_header: A header object.

        revision: One of the constants revisions.SEGY_REVISION_0 or
            revisions.SEGY_REVISION_1

    Returns:
        An integer number of bytes per sample.
    """
    dsf = binary_reel_header.data_sample_format
    seg_y_type = DATA_SAMPLE_FORMAT_TO_SEG_Y_TYPE[dsf]
    ctype = SEG_Y_TYPE_TO_CTYPE[seg_y_type]
    bps = CTYPE_TO_SIZE[ctype]
    return bps


def samples_per_trace(binary_reel_header):
    """Determine the number of samples per trace_samples from the reel header.

    Note: There is no requirement for all traces to be of the same length,
        so this value should be considered indicative only, and as such is
        mostly useful in the absence of other information. The actual number
        of samples for a specific trace_samples should be retrieved from individual
        trace_samples headers.

    Args:
        binary_reel_header: A dictionary containing a reel header, such as obtained
            from read_binary_reel_header()

    Returns:
        An integer number of samples per trace_samples
    """
    return binary_reel_header.num_samples


def trace_length_bytes(binary_reel_header, bps):
    """Determine the trace_samples length in bytes from the reel header.

    Note: There is no requirement for all traces to be of the same length,
        so this value should be considered indicative only, and as such is
        mostly useful in the absence of other information. The actual number
        of samples for a specific trace_samples should be retrieved from individual
        trace_samples headers.

    Args:
        binary_reel_header:  A dictionary containing a reel header, such as obtained
            from read_binary_reel_header()

        bps: The number of bytes per sample, such as obtained from a call to
            bytes_per_sample()

    """
    return samples_per_trace(binary_reel_header) * bps + TRACE_HEADER_NUM_BYTES


def guess_textual_header_encoding(fh):
    fh.seek(0)
    raw_header = fh.read(TEXTUAL_HEADER_NUM_BYTES)
    encoding = guess_encoding(raw_header)
    return encoding


def read_textual_reel_header(fh, encoding):
    """Read the SEG Y card image header, also known as the textual header

    Args:
        fh: A file-like object open in binary mode positioned such that the
            beginning of the textual header will be the next byte to read.

        encoding: Either 'cp037' for EBCDIC or 'ascii' for ASCII.

    Returns:
        A tuple of forty Unicode strings containing the transcoded header data.
    """
    fh.seek(0)
    raw_header = fh.read(TEXTUAL_HEADER_NUM_BYTES)

    num_bytes_read = len(raw_header)
    if num_bytes_read < TEXTUAL_HEADER_NUM_BYTES:
        raise EOFError("Only {} bytes of {} byte textual reel header could be read"
                       .format(num_bytes_read, TEXTUAL_HEADER_NUM_BYTES))

    lines = tuple(bytes(raw_line).decode(encoding) for raw_line in batched(raw_header, CARD_LENGTH))
    return lines


def read_binary_reel_header(fh, endian='>'):
    """Read the SEG Y binary reel header.

    Args:
        fh: A file-like object open in binary mode. Binary header is assumed to
            be at an offset of 3200 bytes from the beginning of the file.

        endian: '>' for big-endian data (the standard and default), '<' for
            little-endian (non-standard)
    """
    header_packer = HeaderPacker(BinaryReelHeader, endian)
    fh.seek(TEXTUAL_HEADER_NUM_BYTES)  # Consider using from_one_based(BinaryReelHeader.START_OFFSET_IN_BYTES)
    buffer = fh.read(BinaryReelHeader.LENGTH_IN_BYTES)
    reel_header = header_packer.unpack(buffer)
    return reel_header


def has_end_text_stanza(ext_header):
    """Determine whether the header is the end text stanza.

    Args:
        ext_header: A sequence of forty CARD_LENGTH character Unicode strings.

    Returns:
        True if the header is the SEG Y Revision 1 end text header,
        otherwise False.
    """
    return END_TEXT_STANZA in ext_header[0]


def read_extended_headers_until_end(fh, encoding):
    """Read an unspecified number of extended textual headers, until the end-text header is found.

    Args:
        fh: A file-like object open in binary mode. The first of any extended textual headers
            is assumed to be at an offset of 3600 bytes from the beginning of the file
            (immediately following the binary reel header).

        encoding: Optional encoding of the header in the file. If None (the
            default) a reliable heuristic will be used to guess the encoding.
            Typically 'cp037' for EBCDIC or 'ascii' for ASCII.

    Returns:
        A list of tuples each containing forty CARD_LENGTH character Unicode strings. If present, the end_text
        stanza is excluded.
    """
    extended_headers = []
    while True:
        ext_header = read_textual_reel_header(fh, encoding)
        extended_headers.append(ext_header)
        if has_end_text_stanza(ext_header):
            break
    return extended_headers


def read_extended_headers_counted(fh, num_expected, encoding):
    """Read a specified number of extended textual headers.

    If an end-text stanza is located prematurely (in anything other than the last expected header)
    reading will be terminated and a warning logged.

    Args:
        fh: A file-like object open in binary mode. The first of any extended textual headers
            is assumed to be at an offset of 3600 bytes from the beginning of the file
            (immediately following the binary reel header).

        num_expected: A non-negative integer of headers.

        encoding: Optional encoding of the header in the file. If None (the
            default) a reliable heuristic will be used to guess the encoding.
            Typically 'cp037' for EBCDIC or 'ascii' for ASCII.

    Returns:
        A list of tuples each containing forty CARD_LENGTH -character Unicode strings.
    """
    assert num_expected >= 0
    extended_headers = []
    for i in range(num_expected):
        ext_header = read_textual_reel_header(fh, encoding)
        if has_end_text_stanza(ext_header):
            if i != num_expected - 1:
                logger().warning("Unexpected end-text extended header.")
            break
        extended_headers.append(ext_header)

    return extended_headers


def read_extended_textual_headers(fh, binary_reel_header, encoding):
    """Read any extended textual reel headers.

    Args:
        fh: A file-like object open in binary mode. The first of any extended textual headers
            is assumed to be at an offset of 3600 bytes from the beginning of the file
            (immediately following the binary reel header).

        binary_reel_header: A dictionary containing data read from the binary
            reel header by the read_binary_reel_header() function.

        encoding: Optional encoding of the header in the file. If None (the
            default) a reliable heuristic will be used to guess the encoding.
            Typically 'cp037' for EBCDIC or 'ascii' for ASCII.

    Returns:
        A sequence of sequences of Unicode strings representing headers of lines of characters. The length of the
        outer sequence will be equal to the number of extended headers read. Each item in the outer sequence will be
        a sequence of exactly forty Unicode strings.  To combine the headers into a single string, consider using
        concatenate_extended_textual_headers().

    Post-condition:
        As a post-condition to this function, the file-pointer of fh will be
        positioned immediately after the last extended textual header, which
        should be the start of the first trace_samples header.
    """
    fh.seek(REEL_HEADER_NUM_BYTES)
    declared_num_ext_headers = num_extended_textual_headers(binary_reel_header)

    if declared_num_ext_headers < 0:
        return read_extended_headers_until_end(fh, encoding)

    return read_extended_headers_counted(fh, declared_num_ext_headers, encoding)


_READ_PROPORTION = 0.75  # The proportion of time spent in catalog_traces
                         # reading the file. Determined empirically.


def catalog_traces(fh, bps, trace_header_format=TraceHeaderRev1, endian='>', progress=None):
    """Build catalogs to facilitate random access to trace_samples data.

    Note:
        This function can take significant time to run, proportional
        to the number of traces in the SEG Y file.

    Four catalogs will be build:

     1. A catalog mapping trace_samples index (0-based) to the position of that
        trace_samples header in the file.

     2. A catalog mapping trace_samples index (0-based) to the number of
        samples in that trace_samples.

     3. A catalog mapping CDP number to the trace_samples index.

     4. A catalog mapping an (inline, crossline) number 2-tuple to
        trace_samples index.

    Args:
        fh: A file-like-object open in binary mode, positioned at the
            start of the first trace_samples header.

        bps: The number of bytes per sample, such as obtained by a call
            to bytes_per_sample()

        trace_header_format: The class defining the trace header format.
            Defaults to TraceHeaderRev1.

        endian: '>' for big-endian data (the standard and default), '<'
            for little-endian (non-standard)

        progress: A unary callable which will be passed a number
            between zero and one indicating the progress made. If
            provided, this callback will be invoked at least once with
            an argument equal to 1

    Returns:
        A 4-tuple of the form (trace_samples-offset-catalog,
                               trace_samples-length-catalog,
                               cdp-catalog,
                               line-catalog)` where
        each catalog is an instance of ``collections.Mapping`` or None
        if no catalog could be built.
    """
    progress_callback = progress if progress is not None else lambda p: None

    if not callable(progress_callback):
        raise TypeError("catalog_traces(): progress callback must be callable")

    trace_header_packer = HeaderPacker(trace_header_format, endian)

    length = file_length(fh)

    pos_begin = fh.tell()

    trace_offset_catalog_builder = CatalogBuilder()
    trace_length_catalog_builder = CatalogBuilder()
    line_catalog_builder = CatalogBuilder()
    alt_line_catalog_builder = CatalogBuilder()
    cdp_catalog_builder = CatalogBuilder()

    for trace_number in count():
        progress_callback(_READ_PROPORTION * pos_begin / length)
        fh.seek(pos_begin)
        data = fh.read(TRACE_HEADER_NUM_BYTES)
        if len(data) < TRACE_HEADER_NUM_BYTES:
            break
        trace_header = trace_header_packer.unpack(data)

        num_samples = trace_header.num_samples
        trace_length_catalog_builder.add(trace_number, num_samples)
        samples_bytes = num_samples * bps
        trace_offset_catalog_builder.add(trace_number, pos_begin)
        # Should we check the data actually exists?
        line_catalog_builder.add((trace_header.inline_number,
                                  trace_header.crossline_number),
                                 trace_number)
        alt_line_catalog_builder.add((trace_header.file_sequence_num,
                                     trace_header.ensemble_num),
                                     trace_number)
        cdp_catalog_builder.add(trace_header.ensemble_num, trace_number)
        pos_end = pos_begin + TRACE_HEADER_NUM_BYTES + samples_bytes
        pos_begin = pos_end

    progress_callback(_READ_PROPORTION)

    trace_offset_catalog = trace_offset_catalog_builder.create()
    progress_callback(_READ_PROPORTION + (_READ_PROPORTION / 4))

    trace_length_catalog = trace_length_catalog_builder.create()
    progress_callback(_READ_PROPORTION + (_READ_PROPORTION / 2))

    cdp_catalog = cdp_catalog_builder.create()
    progress_callback(_READ_PROPORTION + (_READ_PROPORTION * 3 / 4))

    line_catalog = line_catalog_builder.create()

    if line_catalog is None:
        # Some 3D files put Inline and Crossline numbers in (TraceSequenceFile, cdp) pair
        line_catalog = alt_line_catalog_builder.create()

    progress_callback(1)

    return (trace_offset_catalog,
            trace_length_catalog,
            cdp_catalog,
            line_catalog)


def read_trace_header(fh, trace_header_packer, pos=None):
    """Read a trace_samples header.

    Args:
        fh: A file-like-object open in binary mode.

        trace_header_format: A Struct object, such as obtained from a
            call to compile_trace_header_format()

        pos: The file offset in bytes from the beginning from which the data
            is to be read.

    Returns:
        A TraceHeader object.
    """
    if pos is not None:
        fh.seek(pos)
    data = fh.read(TRACE_HEADER_NUM_BYTES)
    # trace_header = TraceHeader._make(
    #     trace_header_format.unpack(data))
    trace_header = trace_header_packer.unpack(data)
    return trace_header


def read_binary_values(fh, pos=None, seg_y_type='int32', count=1, endian='>'):
    """Read a series of values from a binary file.

    Args:
        fh: A file-like-object open in binary mode.

c

        ctype: The SEG Y data type.

        number: The number of items to be read.
    Returns:
        A sequence containing count items.
    """
    ctype = SEG_Y_TYPE_TO_CTYPE[seg_y_type]
    item_size = size_in_bytes(ctype)
    block_size = item_size * count

    fh.seek(pos, os.SEEK_SET)
    buf = fh.read(block_size)

    if len(buf) < block_size:
        raise EOFError("{} bytes requested but only {} available".format(
            block_size, len(buf)))

    values = (unpack_ibm_floats(buf, count)
              if ctype == 'ibm'
              else unpack_values(buf, count, ctype, endian))
    assert len(values) == count
    return values


def unpack_ibm_floats(data, count):
    """Unpack a series of binary-encoded big-endian single-precision IBM floats.

    Args:
        data: A sequence of bytes. (Python 2 - a str object,
            Python 3 - a bytes object)

        count: The number of floats to be read.

    Returns:
        A sequence of floats.
    """
    return [IBMFloat.from_bytes(data[i: i+4]) for i in range(0, count * 4, 4)]


def unpack_values(buf, count, fmt, endian='>'):
    """Unpack a series items from a byte string.

    Args:
        data: A sequence of bytes. (Python 2 - a str object,
            Python 3 - a bytes object)

        count: The number of floats to be read.

        fmt: A format code (one of the values in the datatype.CTYPES
            dictionary)

        endian: '>' for big-endian data (the standard and default), '<'
            for little-endian (non-standard)

    Returns:
        A sequence of objects with type corresponding to the format code.
    """
    a = array(fmt, buf)
    if endian != NATIVE_ENDIANNESS:
        a.byteswap()
    return a


def format_standard_textual_header(revision, **kwargs):
    """Produce a standard SEG Y textual header.

    Args:
        revision: The SEG Y revision.

        **kwargs: Named arguments corresponding to the values in the
            textual_reel_header.TEMPLATE_FIELD_NAMES dictionary,
            which in turn correspond to the placeholders in the
            textual_reel_header.TEMPLATE string.  Any omitted
            arguments will result in placeholders being replaced by spaces.
            If the end_marker argument is not supplied, an appropriate end
            marker will be selected based on the SEG Y revision. For standard
            end markers consider using textual_reel_header.END_TEXTUAL_HEADER
            or textual_reel_header.END_EBCDIC.

    Returns:
        A list of forty Unicode strings.

    Usage:
        header = format_standard_textual_header(1,
                                            client="Lundin",
                                            company="Western Geco",
                                            crew_number=123,
                                            processing1="Sixty North AS",
                                            sweep_start_hz=10,
                                            sweep_end_hz=1000,
                                            sweep_length_ms=10000,
                                            sweep_channel_number=3,
                                            sweep_type='spread')

    """

    kwargs.setdefault('end_marker', textual_reel_header.END_MARKERS[revision])

    template = textual_reel_header.TEMPLATE

    placeholder_slices = parse_template(template)
    background_slices = complementary_intervals(placeholder_slices.values(), 0, len(template))

    chunks = []
    for bg_slice, placeholder in zip_longest(background_slices, placeholder_slices.items()):

        if bg_slice is not None:
            chunks.append(template[bg_slice])

        if placeholder is not None:
            ph_name, ph_slice = placeholder
            ph_arg_name = textual_reel_header.TEMPLATE_FIELD_NAMES[ph_name]
            ph_value = kwargs.pop(ph_arg_name, '')
            ph_len = ph_slice.stop - ph_slice.start
            substitute = str(ph_value)[:ph_len].ljust(ph_len, ' ')
            chunks.append(substitute)

    if len(kwargs) > 0:
        raise TypeError("The following keyword arguments did not correspond to template placeholders: {!r}"
                        .format(list(kwargs.keys())))

    concatenation = ''.join(chunks)
    lines = concatenation.splitlines(keepends=False)

    return lines[1:]  # Omit the first and last lines, which are artifacts of the multi-line string template


_TEMPLATE_PATTERN = r'\{\s*(\w*)\s*\}'
_TEMPLATE_REGEX = re.compile(_TEMPLATE_PATTERN)


def parse_template(template):
    """Parse a template to produce a dictionary of placeholders.

    Args:
        template: The template string containing { field-name } style fixed-width fields.

    Returns:
        A OrderedDict mapping field names to slices objects which can be used to index
        into the template string. The order of the entries is the same as the order within
        which they occur in the template.
    """
    matches = _TEMPLATE_REGEX.finditer(template)

    fields = OrderedDict()
    for match in matches:
        name = match.group(1)
        start = match.start()
        end = match.end()
        fields[name] = slice(start, end)

    return fields


def write_textual_reel_header(fh, lines, encoding):
    """Write the SEG Y card image header, also known as the textual header

    Args:
        fh: A file-like object open in binary mode positioned such that the
            beginning of the textual header will be the next byte to read.

        lines: An iterable series of forty lines, each of which must be a
            Unicode string of CARD_LENGTH characters. The first three characters
            of each line are often "C 1" to "C40" (as required by the SEG Y
            standard) although this is not enforced by this function, since
            many widespread SEG Y readers and writers do not adhere to this
            constraint.  To produce a SEG Y compliant series of header lines
            consider using the format_standard_textual_header() function.

            Any lines longer than CARD_LENGTH characters will be truncated without
            warning.  Any excess lines over CARDS_PER_HEADER will be discarded.  Short
            or omitted lines will be padded with spaces.

        encoding: Typically 'cp037' for EBCDIC or 'ascii' for ASCII.

    Post-condition:
        The file pointer in fh will be positioned at the first byte following the textual
        header.

    Raises:
        UnsupportedEncodingError: If encoding is neither EBCDIC nor ASCII.
        UnicodeError: If the data provided in lines cannot be encoded with the encoding.
    """
    if not is_supported_encoding(encoding):
        raise UnsupportedEncodingError("Writing textual reel header", encoding)

    fh.seek(0)

    padded_lines = [line.encode(encoding).ljust(CARD_LENGTH, ' '.encode(encoding))[:CARD_LENGTH]
                    for line in pad(lines, padding='', size=CARDS_PER_HEADER)]
    joined_header = EMPTY_BYTE_STRING.join(padded_lines)
    assert len(joined_header) == 3200
    fh.write(joined_header)

    fh.seek(TEXTUAL_HEADER_NUM_BYTES)


def write_binary_reel_header(fh, binary_reel_header, endian='>'):
    """Write the binary_reel_header to the given file-like object.

    Args:
        fh: A file-like object open in binary mode for writing.

        binary_reel_header: A header object.

    Post-condition:
        The file pointer for fh will be positioned at the first byte following
        the binary reel header.
    """
    header_packer = HeaderPacker(BinaryReelHeader, endian)  # TODO: Hard wiring
    buffer = header_packer.pack(binary_reel_header)
    fh.write(buffer)
    fh.seek(REEL_HEADER_NUM_BYTES)


def format_extended_textual_header(text, encoding, include_text_stop=False):
    """Format a string into pages and line suitable for an extended textual header.

    Args
        text: An arbitrary text string.  Any universal newlines will be preserved.

        encoding: Either ASCII ('ascii') or EBCDIC ('cp037')

        include_text_stop: If True, a text stop stanza header will be appended, otherwise not.
    """

    if not is_supported_encoding(encoding):
        raise UnsupportedEncodingError("Extended textual header", encoding)

    # According to the standard: "The Extended Textual File Header consists of one or more 3200-byte records, each
    # record containing 40 lines of textual card-image text." It goes on "... Each line in an Extended Textual File
    # Header ends in carriage return and linefeed (EBCDIC 0D25 or ASCII 0D0A)."  Given that we're dealing with fixed-
    # length (80 byte) lines, this implies that we have 78 bytes of space into which we can encode the content of each
    # line, which must be left-justified and padded with spaces.

    width = CARD_LENGTH - len(HEADER_NEWLINE)
    original_lines = text.splitlines()

    # Split overly long lines (i.e. > 78) and pad too-short lines with spaces
    lines = []
    for original_line in original_lines:
        padded_lines = (pad_and_terminate_header_line(original_line[i:i+width], width)
                        for i in range(0, len(original_line), width))
        lines.extend(padded_lines)

    pages = list(batched(lines, 40, pad_and_terminate_header_line('', width)))

    if include_text_stop:
        stop_page = format_extended_textual_header(END_TEXT_STANZA, encoding)[0]
        pages.append(stop_page)

    return pages


def pad_and_terminate_header_line(line, width):
    return line.ljust(width, ' ') + HEADER_NEWLINE


def write_extended_textual_headers(fh, pages, encoding):
    """Write extended textual headers.

    Args:
        fh: A file-like object open in binary mode for writing.

        pages: An iterables series of sequences of Unicode strings, where the outer iterable
            represents 3200 byte pages, each comprised of a sequence of exactly 40 strings of nominally 80 characters
            each.  Although Unicode strings are accepted, and when encoded they should result in exact 80 bytes
            sequences.  To produce a valid data structure for pages, consider using format_extended_textual_header()

        encoding: Either 'cp037' for EBCDIC or 'ascii' for ASCII.

    Post-condition:
        The file pointer in fh will be position at the first byte after the extended textual headers, which is
        also the first byte of the first trace-header.

    Raises:
        ValueError: If the provided header data has the wrong shape.
        UnicodeError: If the textual data could not be encoded into the specified encoding.

    """

    if not is_supported_encoding(encoding):
        raise UnsupportedEncodingError("Writing extended textual header", encoding)

    fh.seek(REEL_HEADER_NUM_BYTES)

    encoded_pages = []
    for page_index, page in enumerate(pages):
        encoded_page = []
        # TODO: Share some of this code with writing the textual reel header.
        for line_index, line in enumerate(page):
            encoded_line = line.encode(encoding)
            num_encoded_bytes = len(encoded_line)
            if num_encoded_bytes != CARD_LENGTH:
                raise ValueError("Extended textual header line {} of page {} at {} bytes is not "
                                 "{} bytes".format(line_index, page_index, num_encoded_bytes, CARD_LENGTH))
            encoded_page.append(encoded_line)
        num_encoded_lines = len(encoded_page)
        if num_encoded_lines != CARDS_PER_HEADER:
            raise ValueError("Extended textual header page {} number of "
                             "lines {} is not {}".format(num_encoded_lines, CARDS_PER_HEADER))
        encoded_pages.append(encoded_page)

    for encoded_page in encoded_pages:
        concatenated_page = EMPTY_BYTE_STRING.join(encoded_page)
        assert(len(concatenated_page) == TEXTUAL_HEADER_NUM_BYTES)
        fh.write(concatenated_page)


def write_trace_header(fh, trace_header, trace_header_packer, pos=None):
    """Write a TraceHeader to file.

    Args:
        fh: A file-like object open in binary mode for writing.

        trace_header: A TraceHeader object.

        trace_header_packer: A Packer object configured for the trace
            header format.

        pos: An optional file offset in bytes from the beginning of the
            file. Defaults to the current file position.
    """
    if pos is not None:
        fh.seek(pos, os.SEEK_SET)

    buf = trace_header_packer.pack(trace_header)
    fh.write(buf)


def write_trace_samples(fh, samples, ctype='l', pos=None, endian='>'):
    """Write a trace samples to a file

    Args:
        fh: A file-like-object open for writing in binary mode.

        values: An iterable series of values.

        ctype: The SEG Y data type.

        pos: An optional offset from the beginning of the file. If omitted,
            any writing is done at the current file position.

        endian: '>' for big-endian data (the standard and default), '<'
            for little-endian (non-standard)
    """
    write_binary_values(fh, samples, ctype, pos, endian)


def write_binary_values(fh, values, ctype, pos=None, endian='>'):
    """Write a series of values to a file.

    Args:
        fh: A file-like-object open for writing in binary mode.

        values: An iterable series of values.

        ctype: The SEG Y data type.

        pos: An optional offset from the beginning of the file. If omitted,
            any writing is done at the current file position.

        endian: '>' for big-endian data (the standard and default), '<'
            for little-endian (non-standard)
    """
    fmt = SEG_Y_TYPE_TO_CTYPE[ctype]

    if pos is not None:
        fh.seek(pos, os.SEEK_SET)

    buf = (pack_ibm_floats(values)
           if fmt == 'ibm'
           else pack_values(values, fmt, endian))

    fh.write(buf)


def pack_ibm_floats(values):
    """Pack floats into binary-encoded big-endian single-precision IBM floats.

    Args:
        values: An iterable series of numeric values.

    Returns:
        A sequence of bytes. (Python 2 - a str object, Python 3 - a bytes
            object)
    """
    return EMPTY_BYTE_STRING.join(bytes(IBMFloat.from_real(value)) for value in values)


def pack_values(values, fmt, endian='>'):
    """Pack values into binary encoded big-endian byte strings.

    Args:
        values: An iterable series of values.

        fmt: A format code (one of the values in the datatype.CTYPES
            dictionary)

        endian: '>' for big-endian data (the standard and default), '<'
            for little-endian (non-standard)
    """
    c_format = '{}{}{}'.format(endian, len(values), fmt)
    return struct.pack(c_format, *values)



#############################################################################################
#                                                                                           #
# Author:   GeoPy Team                                                                      #
# Email:    geopy.info@gmail.com                                                            #
# Date:     September 2018                                                                  #
#                                                                                           #
#############################################################################################

# Create a window for calculating seismic attribute (instantaneous)


from PyQt5 import QtCore, QtGui, QtWidgets
import numpy as np
import os, sys
#
sys.path.append(os.path.dirname(__file__)[:-4][:-4][:-13])
from cognitivegeo.src.seismic.analysis import analysis as seis_ays
from cognitivegeo.src.seismic.attribute import attribute as seis_attrib
from cognitivegeo.src.vis.messager import messager as vis_msg

QtWidgets.QApplication.setAttribute(QtCore.Qt.AA_EnableHighDpiScaling, True)
QtWidgets.QApplication.setAttribute(QtCore.Qt.AA_UseHighDpiPixmaps, True)


class calcinstanattrib(object):

    survinfo = {}
    seisdata = {}
    rootpath = ''
    #
    iconpath = os.path.dirname(__file__)
    dialog = None
    #
    instanattriblist = ['Envelop', 'Quadrature', 'Phase', 'Frequency', 'Cosine of Phase']


    def setupGUI(self, CalcInstanAttrib):
        CalcInstanAttrib.setObjectName("CalcInstanAttrib")
        CalcInstanAttrib.setFixedSize(500, 420)
        icon = QtGui.QIcon()
        icon.addPixmap(QtGui.QPixmap(os.path.join(self.iconpath, "icons/hilbert.png")),
                       QtGui.QIcon.Normal,
                       QtGui.QIcon.Off)
        CalcInstanAttrib.setWindowIcon(icon)
        #
        self.lblproperty = QtWidgets.QLabel(CalcInstanAttrib)
        self.lblproperty.setObjectName("lblproperty")
        self.lblproperty.setGeometry(QtCore.QRect(10, 10, 150, 30))
        self.lwgproperty = QtWidgets.QListWidget(CalcInstanAttrib)
        self.lwgproperty.setObjectName("lwgproperty")
        self.lwgproperty.setGeometry(QtCore.QRect(10, 50, 480, 200))
        self.lwgproperty.setSelectionMode(QtWidgets.QAbstractItemView.SingleSelection)
        self.lblattrib = QtWidgets.QLabel(CalcInstanAttrib)
        self.lblattrib.setObjectName("lblattrib")
        self.lblattrib.setGeometry(QtCore.QRect(10, 270, 230, 30))
        self.cbbattrib = QtWidgets.QComboBox(CalcInstanAttrib)
        self.cbbattrib.setObjectName("cbbattrib")
        self.cbbattrib.setGeometry(QtCore.QRect(10, 310, 230, 30))
        self.lblname = QtWidgets.QLabel(CalcInstanAttrib)
        self.lblname.setObjectName("lblname")
        self.lblname.setGeometry(QtCore.QRect(310, 310, 80, 30))
        self.ldtname = QtWidgets.QLineEdit(CalcInstanAttrib)
        self.ldtname.setObjectName("ldtname")
        self.ldtname.setGeometry(QtCore.QRect(390, 310, 100, 30))
        self.btnapply = QtWidgets.QPushButton(CalcInstanAttrib)
        self.btnapply.setObjectName("btnapply")
        self.btnapply.setGeometry(QtCore.QRect(200, 370, 100, 30))
        icon = QtGui.QIcon()
        icon.addPixmap(QtGui.QPixmap(os.path.join(self.iconpath, "icons/ok.png")),
                       QtGui.QIcon.Normal,
                       QtGui.QIcon.Off)
        self.btnapply.setIcon(icon)
        #
        self.msgbox = QtWidgets.QMessageBox(CalcInstanAttrib)
        self.msgbox.setObjectName("msgbox")
        _center_x = CalcInstanAttrib.geometry().center().x()
        _center_y = CalcInstanAttrib.geometry().center().y()
        self.msgbox.setGeometry(QtCore.QRect(_center_x - 150, _center_y - 50, 300, 100))
        #
        self.retranslateGUI(CalcInstanAttrib)
        QtCore.QMetaObject.connectSlotsByName(CalcInstanAttrib)


    def retranslateGUI(self, CalcInstanAttrib):
        self.dialog = CalcInstanAttrib
        #
        _translate = QtCore.QCoreApplication.translate
        CalcInstanAttrib.setWindowTitle(_translate("CalcInstanAttrib", "Calculate Instantaneous Attribute"))
        self.lblproperty.setText(_translate("CalcInstanAttrib", "Select target property:"))
        self.lblattrib.setText(_translate("CalcInstanAttrib", "Select attribute:"))
        self.cbbattrib.addItems(self.instanattriblist)
        self.cbbattrib.currentIndexChanged.connect(self.changeCbbAttrib)
        self.lblname.setText(_translate("CalcInstanAttrib", "Output name:"))
        self.ldtname.setText(_translate("CalcInstanAttrib", "Envelop"))
        self.btnapply.setText(_translate("CalcInstanAttrib", "Apply"))
        self.btnapply.clicked.connect(self.clickBtnCalcInstanAttrib)
        #
        self.refreshLwgProperty()


    def changeCbbAttrib(self):
        if self.cbbattrib.currentIndex() == 0:
            self.ldtname.setText('Envelop')
        if self.cbbattrib.currentIndex() == 1:
            self.ldtname.setText('Quadrature')
        if self.cbbattrib.currentIndex() == 2:
            self.ldtname.setText('Phase')
        if self.cbbattrib.currentIndex() == 3:
            self.ldtname.setText('Frequency')
        if self.cbbattrib.currentIndex() == 4:
            self.ldtname.setText('CosPhase')


    def clickBtnCalcInstanAttrib(self):
        self.refreshMsgBox()
        #
        _propertylist = self.lwgproperty.selectedItems()
        if len(_propertylist) < 1:
            vis_msg.print("Error in CalcInstanAttrib: No property selected for attribute analysis", type='error')
            QtWidgets.QMessageBox.critical(self.msgbox,
                                           'Calculate Instantaneous Attribute',
                                           'No property selected for attribute analysis')
            return
        if len(self.ldtname.text()) < 1:
            vis_msg.print("Error in CalcInstanAttrib: No name specified for output attribute", type='error')
            QtWidgets.QMessageBox.critical(self.msgbox,
                                           'Calculate Instantaneous Attribute',
                                           'No name specified for output attribute')
            return
        if self.ldtname.text() in self.seisdata.keys() and self.checkSeisData(self.ldtname.text()):
            reply = QtWidgets.QMessageBox.question(self.msgbox, 'Calculate Instantaneous Attribute',
                                                   self.ldtname.text() + ' already exists. Overwrite?',
                                                   QtWidgets.QMessageBox.Yes | QtWidgets.QMessageBox.No,
                                                   QtWidgets.QMessageBox.No)

            if reply == QtWidgets.QMessageBox.No:
                return
        #
        _seisdata = self.seisdata[_propertylist[0].text()]
        #
        _attrib = np.zeros(np.shape(_seisdata))
        if self.cbbattrib.currentIndex() == 0:
            _attrib = seis_attrib.calcInstanEnvelop(_seisdata)
        if self.cbbattrib.currentIndex() == 1:
            _attrib = seis_attrib.calcInstanQuadrature(_seisdata)
        if self.cbbattrib.currentIndex() == 2:
            _attrib = seis_attrib.calcInstanPhase(_seisdata)
        if self.cbbattrib.currentIndex() == 3:
            _attrib = seis_attrib.calcInstanFrequency(_seisdata)
        if self.cbbattrib.currentIndex() == 4:
            _attrib = seis_attrib.calcInstanCosPhase(_seisdata)
        self.seisdata[self.ldtname.text()] = _attrib
        #
        self.refreshLwgProperty()
        #
        QtWidgets.QMessageBox.information(self.msgbox,
                                          "Calculate Instantaneous Attribute",
                                          self.instanattriblist[self.cbbattrib.currentIndex()] + " calculated successfully")
        return


    def refreshLwgProperty(self):
        self.lwgproperty.clear()
        if self.checkSurvInfo() is True:
            for i in sorted(self.seisdata.keys()):
                if self.checkSeisData(i) is True:
                    item = QtWidgets.QListWidgetItem(self.lwgproperty)
                    item.setText(i)
                    self.lwgproperty.addItem(item)


    def refreshMsgBox(self):
        _center_x = self.dialog.geometry().center().x()
        _center_y = self.dialog.geometry().center().y()
        self.msgbox.setGeometry(QtCore.QRect(_center_x - 150, _center_y - 50, 300, 100))


    def checkSurvInfo(self):
        self.refreshMsgBox()
        #
        if seis_ays.checkSeisInfo(self.survinfo) is False:
            # print("CalcInstanAttrib: Survey not found")
            # QtWidgets.QMessageBox.critical(self.msgbox,
            #                                'Calculate Math Attribute from Single Property',
            #                                'Survey not found')
            return False
        return True

    def checkSeisData(self, f):
        self.refreshMsgBox()
        #
        return seis_ays.isSeis3DMatConsistentWithSeisInfo(self.seisdata[f], self.survinfo)


if __name__ == "__main__":
    import sys
    app = QtWidgets.QApplication(sys.argv)
    CalcInstanAttrib = QtWidgets.QWidget()
    gui = calcinstanattrib()
    gui.setupGUI(CalcInstanAttrib)
    CalcInstanAttrib.show()
    sys.exit(app.exec_())


#############################################################################################
#                                                                                           #
# Author:   GeoPy Team                                                                      #
# Email:    geopy.info@gmail.com                                                            #
# Date:     March 2018                                                                      #
#                                                                                           #
#############################################################################################

# Create a GUI for settings (general)

from PyQt5 import QtCore, QtGui, QtWidgets
import os, sys
#
sys.path.append(os.path.dirname(__file__)[:-4][:-4][:-13])
from cognitivegeo.src.core.settings import settings as core_set

QtWidgets.QApplication.setAttribute(QtCore.Qt.AA_EnableHighDpiScaling, True)
QtWidgets.QApplication.setAttribute(QtCore.Qt.AA_UseHighDpiPixmaps, True)


class settingsgeneral(object):

    settings = core_set.General
    #
    iconpath = os.path.dirname(__file__)
    dialog = None


    def setupGUI(self, SettingsGeneral):
        SettingsGeneral.setObjectName("SettingsGeneral")
        SettingsGeneral.setFixedSize(400, 70)
        icon = QtGui.QIcon()
        icon.addPixmap(QtGui.QPixmap(os.path.join(self.iconpath, "icons/settings.png")),
                       QtGui.QIcon.Normal,
                       QtGui.QIcon.Off)
        SettingsGeneral.setWindowIcon(icon)
        #
        self.lblrootpath = QtWidgets.QLabel(SettingsGeneral)
        self.lblrootpath.setObjectName("lblrootpath")
        self.lblrootpath.setGeometry(QtCore.QRect(10, 20, 50, 30))
        self.ldtrootpath = QtWidgets.QLineEdit(SettingsGeneral)
        self.ldtrootpath.setObjectName("ldtrootpath")
        self.ldtrootpath.setGeometry(QtCore.QRect(70, 20, 270, 30))
        self.btnrootpath = QtWidgets.QPushButton(SettingsGeneral)
        self.btnrootpath.setObjectName("btnrootpath")
        self.btnrootpath.setGeometry(QtCore.QRect(350, 20, 40, 30))
        #
        self.msgbox = QtWidgets.QMessageBox(SettingsGeneral)
        self.msgbox.setObjectName("msgbox")
        _center_x = SettingsGeneral.geometry().center().x()
        _center_y = SettingsGeneral.geometry().center().y()
        self.msgbox.setGeometry(QtCore.QRect(_center_x - 150, _center_y - 50, 300, 100))
        #
        self.retranslateGUI(SettingsGeneral)
        QtCore.QMetaObject.connectSlotsByName(SettingsGeneral)


    def retranslateGUI(self, SettingsGeneral):
        self.dialog = SettingsGeneral
        #
        _translate = QtCore.QCoreApplication.translate
        SettingsGeneral.setWindowTitle(_translate("SettingsGeneral", "General Settings"))
        self.lblrootpath.setText(_translate("SettingsGeneral", "Root path:"))
        self.ldtrootpath.setText(_translate("SettingsGeneral", self.settings['RootPath']))
        self.ldtrootpath.setEnabled(False)
        self.btnrootpath.setText(_translate("SettingsGeneral", "Reset"))
        self.btnrootpath.clicked.connect(self.clickBtnRootpath)


    def clickBtnRootpath(self):
        _dialog = QtWidgets.QFileDialog()
        _file = _dialog.getExistingDirectory(None, 'Select Root Path', self.settings['RootPath'],
                                             options=QtWidgets.QFileDialog.DontUseNativeDialog)
        if len(_file) > 0:
            self.ldtrootpath.setText(_file)
            self.settings['RootPath'] = _file


    def refreshMsgBox(self):
        _center_x = self.dialog.geometry().center().x()
        _center_y = self.dialog.geometry().center().y()
        self.msgbox.setGeometry(QtCore.QRect(_center_x - 150, _center_y - 50, 300, 100))


if __name__ == "__main__":
    import sys
    app = QtWidgets.QApplication(sys.argv)
    SettingsGeneral = QtWidgets.QWidget()
    gui = settingsgeneral()
    gui.setupGUI(SettingsGeneral)
    SettingsGeneral.show()
    sys.exit(app.exec_())


SEGY_REVISION_0 = 0
SEGY_REVISION_1 = 1

VARIANTS = {SEGY_REVISION_0: SEGY_REVISION_0,
            SEGY_REVISION_1: SEGY_REVISION_1,
            0: SEGY_REVISION_0,
            1: SEGY_REVISION_1,
            100: SEGY_REVISION_1,
            256: SEGY_REVISION_1}


class SegYRevisionError(Exception):
    pass


def canonicalize_revision(revision):
    """Canonicalize a SEG Y revision.

    Various SEG Y revisions are seen in the wild; this function canonicalizes the supplies revision
    to either SEGY_REVISION_0 or SEGY_REVISION_1.

    Args:
        revision: Any object representing a SEG Y revision.

    Returns:
        Either SEGY_REVISION_0 or SEGY_REVISION_1.

    Raises:
        SegYRevisionError: If the revision is not known.
    """
    try:
        return VARIANTS[revision]
    except KeyError:
        raise SegYRevisionError("Unknown SEG Y Revision {!r}".format(revision))



#############################################################################################
#                                                                                           #
# Author:   GeoPy Team                                                                      #
# Email:    geopy.info@gmail.com                                                            #
# Date:     April 2018                                                                      #
#                                                                                           #
#############################################################################################

# Create a window for viewing seg-y binary header


from PyQt5 import QtCore, QtGui, QtWidgets
import os, sys
#
sys.path.append(os.path.dirname(__file__)[:-4][:-4][:-13])
from cognitivegeo.src.seismic.inputoutput import inputoutput as seis_io

QtWidgets.QApplication.setAttribute(QtCore.Qt.AA_EnableHighDpiScaling, True)
QtWidgets.QApplication.setAttribute(QtCore.Qt.AA_UseHighDpiPixmaps, True)


class viewsegybinaryheader(object):

    segyfile = ""
    #
    iconpath = os.path.dirname(__file__)
    dialog = None


    def setupGUI(self, ViewSegyBinaryHeader):
        ViewSegyBinaryHeader.setObjectName("ViewSegyBinaryHeader")
        ViewSegyBinaryHeader.setFixedSize(510, 460)
        icon = QtGui.QIcon()
        icon.addPixmap(QtGui.QPixmap(os.path.join(self.iconpath, "icons/segy.png")),
                       QtGui.QIcon.Normal,
                       QtGui.QIcon.Off)
        ViewSegyBinaryHeader.setWindowIcon(icon)
        #
        self.lblhelp = QtWidgets.QLabel(ViewSegyBinaryHeader)
        self.lblhelp.setObjectName("lblhelp")
        self.lblhelp.setGeometry(QtCore.QRect(410, 10, 80, 30))
        self.twgheader = QtWidgets.QTableWidget(ViewSegyBinaryHeader)
        self.twgheader.setObjectName("twgheader")
        self.twgheader.setGeometry(QtCore.QRect(10, 50, 480, 380))
        self.twgheader.setSelectionMode(QtWidgets.QAbstractItemView.NoSelection)
        self.twgheader.setRowCount(30)
        self.twgheader.setColumnCount(1)
        #
        self.msgbox = QtWidgets.QMessageBox(ViewSegyBinaryHeader)
        self.msgbox.setObjectName("msgbox")
        _center_x = ViewSegyBinaryHeader.geometry().center().x()
        _center_y = ViewSegyBinaryHeader.geometry().center().y()
        self.msgbox.setGeometry(QtCore.QRect(_center_x - 150, _center_y - 50, 300, 100))
        #
        self.retranslateGUI(ViewSegyBinaryHeader)
        QtCore.QMetaObject.connectSlotsByName(ViewSegyBinaryHeader)



    def retranslateGUI(self, ViewSegyBinaryHeader):
        self.dialog = ViewSegyBinaryHeader
        #
        _translate = QtCore.QCoreApplication.translate
        ViewSegyBinaryHeader.setWindowTitle(_translate("ViewSegyBinaryHeader", "SEG-Y Binary Header"))
        self.lblhelp.setText(_translate("ViewSegyBinaryHeader",
                                        '<a href="https://seg.org/Portals/0/SEG/News%20and%20Resources/Technical%20Standards/seg_y_rev1.pdf">SEG-Y Format</a>'))
        self.lblhelp.setOpenExternalLinks(True)
        _rowheader = ['job_id_num (3201-3204)', 'line_num (3205-3208)',
                      'reel_num (3209-3212)', 'data_traces_per_ensemble (3213-3214)',
                      'auxiliary_traces_per_ensemble (3215-3216)', 'sample_interval (3217-3218)',
                      'original_field_sample_interval (3219-3220)', 'num_samples (3221-3222)',
                      'original_field_num_samples (3223-3224)', 'data_sample_format (3225-3226)',
                      'ensemble_fold (3227-3228)', 'trace_sorting (3229-3230)',
                      'vertical_sum_code (3231-3232)', 'sweep_frequency_at_start (3233-3234)',
                      'sweep_frequency_at_end (3235-3236)', 'sweep_length (3237-3238)',
                      'sweep_type (3239-3240)', 'sweep_trace_number (3241-3242)',
                      'sweep_trace_taper_length_at_start (3243-3244)', 'sweep_trace_taper_length_at_end (3245-3246)',
                      'taper_type (3247-3248)', 'correlated_data_traces (3249-3250)',
                      'binary_gain_recovered (3251-3252)', 'amplitude_recovery_method (3253-3254)',
                      'measurement_system (3255-3256)', 'impulse_signal_polarity (3257-3258)',
                      'vibratory_polarity_code (3259-3260)', 'format_revision_num (3501-3502)',
                      'fixed_length_trace_flag (3503-3504)', 'num_extended_textual_headers (3505-3506)']
        self.twgheader.setVerticalHeaderLabels(_rowheader)
        self.twgheader.horizontalHeader().hide()
        #
        if os.path.exists(self.segyfile):
            _header = seis_io.readSegyBinaryHeader(self.segyfile)
            # job_id_num
            _item = QtWidgets.QTableWidgetItem()
            _item.setText(_translate("ViewSegyBinaryHeader", str(_header.job_id_num)))
            _item.setFlags(QtCore.Qt.ItemIsEditable)
            _item.setTextAlignment(QtCore.Qt.AlignCenter)
            self.twgheader.setItem(0, 0, _item)
            # line_num
            _item = QtWidgets.QTableWidgetItem()
            _item.setText(_translate("ViewSegyBinaryHeader", str(_header.line_num)))
            _item.setFlags(QtCore.Qt.ItemIsEditable)
            _item.setTextAlignment(QtCore.Qt.AlignCenter)
            self.twgheader.setItem(1, 0, _item)
            # reel_num
            _item = QtWidgets.QTableWidgetItem()
            _item.setText(_translate("ViewSegyBinaryHeader", str(_header.reel_num)))
            _item.setFlags(QtCore.Qt.ItemIsEditable)
            _item.setTextAlignment(QtCore.Qt.AlignCenter)
            self.twgheader.setItem(2, 0, _item)
            # data_traces_per_ensemble
            _item = QtWidgets.QTableWidgetItem()
            _item.setText(_translate("ViewSegyBinaryHeader", str(_header.data_traces_per_ensemble)))
            _item.setFlags(QtCore.Qt.ItemIsEditable)
            _item.setTextAlignment(QtCore.Qt.AlignCenter)
            self.twgheader.setItem(3, 0, _item)
            # auxiliary_traces_per_ensemble
            _item = QtWidgets.QTableWidgetItem()
            _item.setText(_translate("ViewSegyBinaryHeader", str(_header.auxiliary_traces_per_ensemble)))
            _item.setFlags(QtCore.Qt.ItemIsEditable)
            _item.setTextAlignment(QtCore.Qt.AlignCenter)
            self.twgheader.setItem(4, 0, _item)
            # sample_interval
            _item = QtWidgets.QTableWidgetItem()
            _item.setText(_translate("ViewSegyBinaryHeader", str(_header.sample_interval)))
            _item.setFlags(QtCore.Qt.ItemIsEditable)
            _item.setTextAlignment(QtCore.Qt.AlignCenter)
            self.twgheader.setItem(5, 0, _item)
            # original_field_sample_interval
            _item = QtWidgets.QTableWidgetItem()
            _item.setText(_translate("ViewSegyBinaryHeader", str(_header.original_field_sample_interval)))
            _item.setFlags(QtCore.Qt.ItemIsEditable)
            _item.setTextAlignment(QtCore.Qt.AlignCenter)
            self.twgheader.setItem(6, 0, _item)
            # num_samples
            _item = QtWidgets.QTableWidgetItem()
            _item.setText(_translate("ViewSegyBinaryHeader", str(_header.num_samples)))
            _item.setFlags(QtCore.Qt.ItemIsEditable)
            _item.setTextAlignment(QtCore.Qt.AlignCenter)
            self.twgheader.setItem(7, 0, _item)
            # original_field_num_samples
            _item = QtWidgets.QTableWidgetItem()
            _item.setText(_translate("ViewSegyBinaryHeader", str(_header.original_field_num_samples)))
            _item.setFlags(QtCore.Qt.ItemIsEditable)
            _item.setTextAlignment(QtCore.Qt.AlignCenter)
            self.twgheader.setItem(8, 0, _item)
            # data_sample_format
            _item = QtWidgets.QTableWidgetItem()
            _item.setText(_translate("ViewSegyBinaryHeader", str(_header.data_sample_format)))
            _item.setFlags(QtCore.Qt.ItemIsEditable)
            _item.setTextAlignment(QtCore.Qt.AlignCenter)
            self.twgheader.setItem(9, 0, _item)
            # ensemble_fold
            _item = QtWidgets.QTableWidgetItem()
            _item.setText(_translate("ViewSegyBinaryHeader", str(_header.ensemble_fold)))
            _item.setFlags(QtCore.Qt.ItemIsEditable)
            _item.setTextAlignment(QtCore.Qt.AlignCenter)
            self.twgheader.setItem(10, 0, _item)
            # trace_sorting
            _item = QtWidgets.QTableWidgetItem()
            _item.setText(_translate("ViewSegyBinaryHeader", str(_header.trace_sorting)))
            _item.setFlags(QtCore.Qt.ItemIsEditable)
            _item.setTextAlignment(QtCore.Qt.AlignCenter)
            self.twgheader.setItem(11, 0, _item)
            # vertical_sum_code
            _item = QtWidgets.QTableWidgetItem()
            _item.setText(_translate("ViewSegyBinaryHeader", str(_header.vertical_sum_code)))
            _item.setFlags(QtCore.Qt.ItemIsEditable)
            _item.setTextAlignment(QtCore.Qt.AlignCenter)
            self.twgheader.setItem(12, 0, _item)
            # sweep_frequency_at_start
            _item = QtWidgets.QTableWidgetItem()
            _item.setText(_translate("ViewSegyBinaryHeader", str(_header.sweep_frequency_at_start)))
            _item.setFlags(QtCore.Qt.ItemIsEditable)
            _item.setTextAlignment(QtCore.Qt.AlignCenter)
            self.twgheader.setItem(13, 0, _item)
            # sweep_frequency_at_end
            _item = QtWidgets.QTableWidgetItem()
            _item.setText(_translate("ViewSegyBinaryHeader", str(_header.sweep_frequency_at_end)))
            _item.setFlags(QtCore.Qt.ItemIsEditable)
            _item.setTextAlignment(QtCore.Qt.AlignCenter)
            self.twgheader.setItem(14, 0, _item)
            # sweep_length
            _item = QtWidgets.QTableWidgetItem()
            _item.setText(_translate("ViewSegyBinaryHeader", str(_header.sweep_length)))
            _item.setFlags(QtCore.Qt.ItemIsEditable)
            _item.setTextAlignment(QtCore.Qt.AlignCenter)
            self.twgheader.setItem(15, 0, _item)
            # sweep_type
            _item = QtWidgets.QTableWidgetItem()
            _item.setText(_translate("ViewSegyBinaryHeader", str(_header.sweep_type)))
            _item.setFlags(QtCore.Qt.ItemIsEditable)
            _item.setTextAlignment(QtCore.Qt.AlignCenter)
            self.twgheader.setItem(16, 0, _item)
            # sweep_trace_number
            _item = QtWidgets.QTableWidgetItem()
            _item.setText(_translate("ViewSegyBinaryHeader", str(_header.sweep_trace_number)))
            _item.setFlags(QtCore.Qt.ItemIsEditable)
            _item.setTextAlignment(QtCore.Qt.AlignCenter)
            self.twgheader.setItem(17, 0, _item)
            # sweep_trace_taper_length_at_start
            _item = QtWidgets.QTableWidgetItem()
            _item.setText(_translate("ViewSegyBinaryHeader", str(_header.sweep_trace_taper_length_at_start)))
            _item.setFlags(QtCore.Qt.ItemIsEditable)
            _item.setTextAlignment(QtCore.Qt.AlignCenter)
            self.twgheader.setItem(18, 0, _item)
            # sweep_trace_taper_length_at_end
            _item = QtWidgets.QTableWidgetItem()
            _item.setText(_translate("ViewSegyBinaryHeader", str(_header.sweep_trace_taper_length_at_end)))
            _item.setFlags(QtCore.Qt.ItemIsEditable)
            _item.setTextAlignment(QtCore.Qt.AlignCenter)
            self.twgheader.setItem(19, 0, _item)
            # taper_type
            _item = QtWidgets.QTableWidgetItem()
            _item.setText(_translate("ViewSegyBinaryHeader", str(_header.taper_type)))
            _item.setFlags(QtCore.Qt.ItemIsEditable)
            _item.setTextAlignment(QtCore.Qt.AlignCenter)
            self.twgheader.setItem(20, 0, _item)
            # correlated_data_traces
            _item = QtWidgets.QTableWidgetItem()
            _item.setText(_translate("ViewSegyBinaryHeader", str(_header.correlated_data_traces)))
            _item.setFlags(QtCore.Qt.ItemIsEditable)
            _item.setTextAlignment(QtCore.Qt.AlignCenter)
            self.twgheader.setItem(21, 0, _item)
            # binary_gain_recovered
            _item = QtWidgets.QTableWidgetItem()
            _item.setText(_translate("ViewSegyBinaryHeader", str(_header.binary_gain_recovered)))
            _item.setFlags(QtCore.Qt.ItemIsEditable)
            _item.setTextAlignment(QtCore.Qt.AlignCenter)
            self.twgheader.setItem(22, 0, _item)
            # amplitude_recovery_method
            _item = QtWidgets.QTableWidgetItem()
            _item.setText(_translate("ViewSegyBinaryHeader", str(_header.amplitude_recovery_method)))
            _item.setFlags(QtCore.Qt.ItemIsEditable)
            _item.setTextAlignment(QtCore.Qt.AlignCenter)
            self.twgheader.setItem(23, 0, _item)
            # measurement_system
            _item = QtWidgets.QTableWidgetItem()
            _item.setText(_translate("ViewSegyBinaryHeader", str(_header.measurement_system)))
            _item.setFlags(QtCore.Qt.ItemIsEditable)
            _item.setTextAlignment(QtCore.Qt.AlignCenter)
            self.twgheader.setItem(24, 0, _item)
            # impulse_signal_polarity
            _item = QtWidgets.QTableWidgetItem()
            _item.setText(_translate("ViewSegyBinaryHeader", str(_header.impulse_signal_polarity)))
            _item.setFlags(QtCore.Qt.ItemIsEditable)
            _item.setTextAlignment(QtCore.Qt.AlignCenter)
            self.twgheader.setItem(25, 0, _item)
            # vibratory_polarity_code
            _item = QtWidgets.QTableWidgetItem()
            _item.setText(_translate("ViewSegyBinaryHeader", str(_header.vibratory_polarity_code)))
            _item.setFlags(QtCore.Qt.ItemIsEditable)
            _item.setTextAlignment(QtCore.Qt.AlignCenter)
            self.twgheader.setItem(26, 0, _item)
            # format_revision_num
            _item = QtWidgets.QTableWidgetItem()
            _item.setText(_translate("ViewSegyBinaryHeader", str(_header.format_revision_num)))
            _item.setFlags(QtCore.Qt.ItemIsEditable)
            _item.setTextAlignment(QtCore.Qt.AlignCenter)
            self.twgheader.setItem(27, 0, _item)
            # fixed_length_trace_flag
            _item = QtWidgets.QTableWidgetItem()
            _item.setText(_translate("ViewSegyBinaryHeader", str(_header.fixed_length_trace_flag)))
            _item.setFlags(QtCore.Qt.ItemIsEditable)
            _item.setTextAlignment(QtCore.Qt.AlignCenter)
            self.twgheader.setItem(28, 0, _item)
            # num_extended_textual_headers
            _item = QtWidgets.QTableWidgetItem()
            _item.setText(_translate("ViewSegyBinaryHeader", str(_header.num_extended_textual_headers)))
            _item.setFlags(QtCore.Qt.ItemIsEditable)
            _item.setTextAlignment(QtCore.Qt.AlignCenter)
            self.twgheader.setItem(29, 0, _item)


    def refreshMsgBox(self):
        _center_x = self.dialog.geometry().center().x()
        _center_y = self.dialog.geometry().center().y()
        self.msgbox.setGeometry(QtCore.QRect(_center_x - 150, _center_y - 50, 300, 100))


if __name__ == "__main__":
    import sys
    app = QtWidgets.QApplication(sys.argv)
    ViewSegyBinaryHeader = QtWidgets.QWidget()
    gui = viewsegybinaryheader()
    gui.setupGUI(ViewSegyBinaryHeader)
    ViewSegyBinaryHeader.show()
    sys.exit(app.exec_())


#############################################################################################
#                                                                                           #
# Author:   GeoPy Team                                                                      #
# Email:    geopy.info@gmail.com                                                            #
# Date:     November 2019                                                                   #
#                                                                                           #
#############################################################################################

# Create a GUI for settings (3d viewer)

from PyQt5 import QtCore, QtGui, QtWidgets
import os, sys
#
sys.path.append(os.path.dirname(__file__)[:-4][:-4][:-13])
from cognitivegeo.src.core.settings import settings as core_set
from cognitivegeo.src.vis.viewer3d import viewer3d as vis_viewer3d
from cognitivegeo.src.vis.player import player as vis_player

QtWidgets.QApplication.setAttribute(QtCore.Qt.AA_EnableHighDpiScaling, True)
QtWidgets.QApplication.setAttribute(QtCore.Qt.AA_UseHighDpiPixmaps, True)


class settingsviewer(object):

    settings = core_set.Viewer
    #
    iconpath = os.path.dirname(__file__)
    dialog = None


    def setupGUI(self, SettingsViewer):
        SettingsViewer.setObjectName("SettingsViewer")
        SettingsViewer.setFixedSize(840, 310)
        icon = QtGui.QIcon()
        icon.addPixmap(QtGui.QPixmap(os.path.join(self.iconpath, "icons/dice.png")),
                       QtGui.QIcon.Normal,
                       QtGui.QIcon.Off)
        SettingsViewer.setWindowIcon(icon)
        #
        self.lblviewer3d = QtWidgets.QLabel(SettingsViewer)
        self.lblviewer3d.setObjectName("lblviewer3d")
        self.lblviewer3d.setGeometry(QtCore.QRect(10, 20, 50, 30))
        #
        self.lblgohome = QtWidgets.QLabel(SettingsViewer)
        self.lblgohome.setObjectName("lblgohome")
        self.lblgohome.setGeometry(QtCore.QRect(30, 60, 60, 30))
        self.cbbgohome = QtWidgets.QComboBox(SettingsViewer)
        self.cbbgohome.setObjectName("cbbgohome")
        self.cbbgohome.setGeometry(QtCore.QRect(100, 60, 100, 30))
        #
        self.lblviewfrom = QtWidgets.QLabel(SettingsViewer)
        self.lblviewfrom.setObjectName("lblviewfrom")
        self.lblviewfrom.setGeometry(QtCore.QRect(450, 60, 60, 30))
        self.twgviewfrom = QtWidgets.QTableWidget(SettingsViewer)
        self.twgviewfrom.setObjectName("twgplayer")
        self.twgviewfrom.setGeometry(QtCore.QRect(520, 60, 310, 50))
        self.twgviewfrom.setColumnCount(3)
        self.twgviewfrom.setRowCount(1)
        self.twgviewfrom.setSelectionMode(QtWidgets.QAbstractItemView.NoSelection)
        self.twgviewfrom.verticalHeader().hide()
        #
        self.lblplayer = QtWidgets.QLabel(SettingsViewer)
        self.lblplayer.setObjectName("lblplayer")
        self.lblplayer.setGeometry(QtCore.QRect(10, 130, 160, 30))
        self.twgplayer = QtWidgets.QTableWidget(SettingsViewer)
        self.twgplayer.setObjectName("twgplayer")
        self.twgplayer.setGeometry(QtCore.QRect(30, 170, 800, 70))
        self.twgplayer.setColumnCount(8)
        self.twgplayer.setRowCount(1)
        self.twgplayer.setSelectionMode(QtWidgets.QAbstractItemView.NoSelection)
        self.twgplayer.verticalHeader().hide()
        #
        self.btnapply = QtWidgets.QPushButton(SettingsViewer)
        self.btnapply.setObjectName("btnapply")
        self.btnapply.setGeometry(QtCore.QRect(370, 260, 100, 30))
        icon = QtGui.QIcon()
        icon.addPixmap(QtGui.QPixmap(os.path.join(self.iconpath, "icons/ok.png")),
                       QtGui.QIcon.Normal,
                       QtGui.QIcon.Off)
        self.btnapply.setIcon(icon)
        #
        self.msgbox = QtWidgets.QMessageBox(SettingsViewer)
        self.msgbox.setObjectName("msgbox")
        _center_x = SettingsViewer.geometry().center().x()
        _center_y = SettingsViewer.geometry().center().y()
        self.msgbox.setGeometry(QtCore.QRect(_center_x - 150, _center_y - 50, 300, 100))
        #
        self.retranslateGUI(SettingsViewer)
        QtCore.QMetaObject.connectSlotsByName(SettingsViewer)


    def retranslateGUI(self, SettingsViewer):
        self.dialog = SettingsViewer
        #
        _translate = QtCore.QCoreApplication.translate
        SettingsViewer.setWindowTitle(_translate("SettingsViewer", "Viewer Settings"))
        self.lblviewer3d.setText(_translate("SettingsViewer", "3-D:"))
        self.lblgohome.setText(_translate("SettingsViewer", "Go Home:"))
        self.cbbgohome.addItems(vis_viewer3d.GoHomeKeyList)
        #
        self.lblviewfrom.setText(_translate("SettingsViewer", "View from:"))
        self.twgviewfrom.setHorizontalHeaderLabels(['Inline', "Crossline", 'Time/depth'])
        for _i in vis_viewer3d.ViewFromPropertyList:
            _idx = list.index(vis_viewer3d.ViewFromPropertyList, _i)
            item = QtWidgets.QComboBox()
            item.addItems(vis_viewer3d.ViewFromKeyList)
            self.twgviewfrom.setCellWidget(0, _idx, item)
        #
        self.lblplayer.setText(_translate("SettingsViewer", "Player:"))
        self.twgplayer.setHorizontalHeaderLabels(['|<--', "<<--", '<<<<', '>>||',
                                                  '>>>>', '-->>', '-->|', 'Interval'])
        for _i in vis_player.PlayerPropertyList[:-1]:
            _idx = list.index(vis_player.PlayerPropertyList, _i)
            item = QtWidgets.QComboBox()
            item.addItems(vis_player.PlayerKeyList)
            self.twgplayer.setCellWidget(0, _idx, item)
        _idx = list.index(vis_player.PlayerPropertyList, vis_player.PlayerPropertyList[-1])
        item = QtWidgets.QComboBox()
        item.addItems([str(t) for t in vis_player.PlayerIntervalList])
        self.twgplayer.setCellWidget(0, _idx, item)
        #
        self.btnapply.setText(_translate("SettingsViewer", "Apply"))
        self.btnapply.clicked.connect(self.clickBtnApply)
        #
        self.cbbgohome.setCurrentIndex(list.index(vis_viewer3d.GoHomeKeyList, self.settings['Viewer3D']['GoHome']))
        #
        for _i in vis_viewer3d.ViewFromPropertyList:
            _idx = list.index(vis_viewer3d.ViewFromPropertyList, _i)
            self.twgviewfrom.cellWidget(0, _idx).setCurrentIndex(list.index(vis_viewer3d.ViewFromKeyList,
                                                                            self.settings['Viewer3D']['ViewFrom'][_i]))
        #
        for _i in vis_player.PlayerPropertyList[:-1]:
            _idx = list.index(vis_player.PlayerPropertyList, _i)
            self.twgplayer.cellWidget(0, _idx).setCurrentIndex(list.index(vis_player.PlayerKeyList,
                                                                          self.settings['Player'][_i]))
        _i = vis_player.PlayerPropertyList[-1]
        _idx = list.index(vis_player.PlayerPropertyList, _i)
        self.twgplayer.cellWidget(0, _idx).setCurrentIndex(list.index(vis_player.PlayerIntervalList,
                                                                      self.settings['Player'][_i]))


    def clickBtnApply(self):
        self.refreshMsgBox()
        #
        self.settings['Viewer3D']['GoHome'] = vis_viewer3d.GoHomeKeyList[self.cbbgohome.currentIndex()]
        #
        for _prop in vis_viewer3d.ViewFromPropertyList:
            _idx = list.index(vis_viewer3d.ViewFromPropertyList, _prop)
            self.settings['Viewer3D']['ViewFrom'][_prop] = \
                vis_viewer3d.ViewFromKeyList[self.twgviewfrom.cellWidget(0, _idx).currentIndex()]
        #
        for _prop in vis_player.PlayerPropertyList[:-1]:
            _idx = list.index(vis_player.PlayerPropertyList, _prop)
            self.settings['Player'][_prop] = \
                vis_player.PlayerKeyList[self.twgplayer.cellWidget(0, _idx).currentIndex()]
        _prop = vis_player.PlayerPropertyList[-1]
        _idx = list.index(vis_player.PlayerPropertyList, _prop)
        self.settings['Player'][_prop] = \
            vis_player.PlayerIntervalList[self.twgplayer.cellWidget(0, _idx).currentIndex()]
        #
        self.dialog.close()


    def refreshMsgBox(self):
        _center_x = self.dialog.geometry().center().x()
        _center_y = self.dialog.geometry().center().y()
        self.msgbox.setGeometry(QtCore.QRect(_center_x - 150, _center_y - 50, 300, 100))


if __name__ == "__main__":
    import sys
    app = QtWidgets.QApplication(sys.argv)
    SettingsViewer = QtWidgets.QWidget()
    gui = settingsviewer()
    gui.setupGUI(SettingsViewer)
    SettingsViewer.show()
    sys.exit(app.exec_())


import pkg_resources

loaded = set()


def load_entry_points(name=None):
    """Load extension packages into the segpy.ext namespace.

    Any packages registered against the 'segpy.ext' entry-point group will be
    installed dynamically into the segpy.ext namespace.
    """
    for entry_point in pkg_resources.iter_entry_points(group='segpy.ext', name=name):
        package = entry_point.load()
        if package not in loaded:
            loaded.add(package)
            __path__.extend(package.__path__)
            package.load()


load_entry_points()



#############################################################################################
#                                                                                           #
# Author:   GeoPy Team                                                                      #
# Email:    geopy.info@gmail.com                                                            #
# Date:     August 2018                                                                     #
#                                                                                           #
#############################################################################################

# Create a window for managing pointsets


from PyQt5 import QtCore, QtGui, QtWidgets
import os, sys
#
sys.path.append(os.path.dirname(__file__)[:-4][:-4][:-13])
from cognitivegeo.src.basic.matdict import matdict as basic_mdt
from cognitivegeo.src.core.settings import settings as core_set
from cognitivegeo.src.pointset.analysis import analysis as point_ays
from cognitivegeo.src.gui.editpointset import editpointset as gui_editpointset
from cognitivegeo.src.vis.messager import messager as vis_msg

QtWidgets.QApplication.setAttribute(QtCore.Qt.AA_EnableHighDpiScaling, True)
QtWidgets.QApplication.setAttribute(QtCore.Qt.AA_UseHighDpiPixmaps, True)


class managepointset(object):

    pointsetdata = {}
    rootpath = ''
    linestyle = core_set.Visual['Line']
    fontstyle = core_set.Visual['Font']
    #
    iconpath = os.path.dirname(__file__)
    dialog = None


    def setupGUI(self, ManagePointSet):
        ManagePointSet.setObjectName("ManagePointSet")
        ManagePointSet.setFixedSize(320, 420)
        icon = QtGui.QIcon()
        icon.addPixmap(QtGui.QPixmap(os.path.join(self.iconpath, "icons/point.png")),
                       QtGui.QIcon.Normal,
                       QtGui.QIcon.Off)
        ManagePointSet.setWindowIcon(icon)
        #
        self.lblpoint = QtWidgets.QLabel(ManagePointSet)
        self.lblpoint.setObjectName("lblpoint")
        self.lblpoint.setGeometry(QtCore.QRect(10, 10, 150, 30))
        self.twgpoint = QtWidgets.QTableWidget(ManagePointSet)
        self.twgpoint.setObjectName("twgpoint")
        self.twgpoint.setGeometry(QtCore.QRect(10, 50, 300, 300))
        self.twgpoint.setColumnCount(2)
        self.twgpoint.setSelectionMode(QtWidgets.QAbstractItemView.NoSelection)
        self.twgpoint.verticalHeader().hide()
        self.btnedit = QtWidgets.QPushButton(ManagePointSet)
        self.btnedit.setObjectName("btnedit")
        self.btnedit.setGeometry(QtCore.QRect(210, 360, 100, 30))
        icon = QtGui.QIcon()
        icon.addPixmap(QtGui.QPixmap(os.path.join(self.iconpath, "icons/pen.png")),
                       QtGui.QIcon.Normal,
                       QtGui.QIcon.Off)
        self.btnedit.setIcon(icon)
        #
        self.msgbox = QtWidgets.QMessageBox(ManagePointSet)
        self.msgbox.setObjectName("msgbox")
        _center_x = ManagePointSet.geometry().center().x()
        _center_y = ManagePointSet.geometry().center().y()
        self.msgbox.setGeometry(QtCore.QRect(_center_x - 150, _center_y - 50, 300, 100))
        #
        self.retranslateGUI(ManagePointSet)
        QtCore.QMetaObject.connectSlotsByName(ManagePointSet)


    def retranslateGUI(self, ManagePointSet):
        self.dialog = ManagePointSet
        #
        _translate = QtCore.QCoreApplication.translate
        ManagePointSet.setWindowTitle(_translate("ManagePointSet", "Manage PointSet"))
        self.lblpoint.setText(_translate("ManagePointSet", "Available pointsets:"))
        self.btnedit.setText(_translate("ManagePointSet", "Edit"))
        self.btnedit.setToolTip("Edit pointsets")
        self.btnedit.clicked.connect(self.clickBtnEdit)
        #
        self.refreshTwgPoint()

    def clickBtnEdit(self):
        _edit = QtWidgets.QDialog()
        _gui = gui_editpointset()
        _gui.pointsetdata = self.pointsetdata
        _gui.rootpath = self.rootpath
        _gui.linestyle = self.linestyle
        _gui.fontstyle = self.fontstyle
        _gui.setupGUI(_edit)
        _edit.exec()
        self.pointsetdata = _gui.pointsetdata
        _edit.show()
        #
        self.refreshTwgPoint()


    def refreshTwgPoint(self):
        self.twgpoint.clear()
        self.twgpoint.setHorizontalHeaderLabels(["Name", "Length"])
        if len(self.pointsetdata.keys()) > 0:
            _idx = 0
            self.twgpoint.setRowCount(len(self.pointsetdata.keys()))
            for i in sorted(self.pointsetdata.keys()):
                if self.checkPointSet(i):
                    item = QtWidgets.QTableWidgetItem()
                    item.setText(i)
                    item.setFlags(QtCore.Qt.ItemIsSelectable | QtCore.Qt.ItemIsEnabled)
                    self.twgpoint.setItem(_idx, 0, item)
                    item = QtWidgets.QTableWidgetItem()
                    item.setText(str(basic_mdt.maxDictConstantRow(self.pointsetdata[i])))
                    item.setTextAlignment(QtCore.Qt.AlignCenter)
                    item.setFlags(QtCore.Qt.ItemIsEditable)
                    self.twgpoint.setItem(_idx, 1, item)
                    _idx = _idx + 1
            self.twgpoint.setRowCount(_idx)


    def refreshMsgBox(self):
        _center_x = self.dialog.geometry().center().x()
        _center_y = self.dialog.geometry().center().y()
        self.msgbox.setGeometry(QtCore.QRect(_center_x - 150, _center_y - 50, 300, 100))


    def checkPointSet(self, name):
        return point_ays.checkPoint(self.pointsetdata[name])


if __name__ == "__main__":
    import sys
    app = QtWidgets.QApplication(sys.argv)
    ManagePointSet = QtWidgets.QWidget()
    gui = managepointset()
    gui.setupGUI(ManagePointSet)
    ManagePointSet.show()
    sys.exit(app.exec_())


#############################################################################################
#                                                                                           #
# Author:   GeoPy Team                                                                      #
# Email:    geopy.info@gmail.com                                                            #
# Date:     January 2019                                                                    #
#                                                                                           #
#############################################################################################

# Create a window for plot pre-stack seismic shots


from PyQt5 import QtCore, QtGui, QtWidgets
import numpy as np
import sys, os
#
sys.path.append(os.path.dirname(__file__)[:-4][:-4][:-13])
from cognitivegeo.src.basic.data import data as basic_data
from cognitivegeo.src.core.settings import settings as core_set
from cognitivegeo.src.psseismic.analysis import analysis as psseis_ays
from cognitivegeo.src.psseismic.visualization import visualization as psseis_vis
from cognitivegeo.src.vis.colormap import colormap as vis_cmap
from cognitivegeo.src.gui.configplayer import configplayer as gui_configplayer
from cognitivegeo.src.vis.messager import messager as vis_msg

QtWidgets.QApplication.setAttribute(QtCore.Qt.AA_EnableHighDpiScaling, True)
QtWidgets.QApplication.setAttribute(QtCore.Qt.AA_UseHighDpiPixmaps, True)


class plotvis2dpsseisshot(object):

    psseisdata = {}
    plotstyle = core_set.Visual['Image']
    playerconfig = core_set.Viewer['Player']
    fontstyle = core_set.Visual['Font']
    #
    iconpath = os.path.dirname(__file__)
    dialog = None

    def setupGUI(self, PlotVis2DPsSeisShot):
        PlotVis2DPsSeisShot.setObjectName("PlotVis2DPsSeisShot")
        PlotVis2DPsSeisShot.setFixedSize(500, 320)
        icon = QtGui.QIcon()
        icon.addPixmap(QtGui.QPixmap(os.path.join(self.iconpath, "icons/gather.png")),
                       QtGui.QIcon.Normal,
                       QtGui.QIcon.Off)
        PlotVis2DPsSeisShot.setWindowIcon(icon)
        #
        self.lwgpsseis = QtWidgets.QListWidget(PlotVis2DPsSeisShot)
        self.lwgpsseis.setObjectName("lwgpsseis")
        self.lwgpsseis.setGeometry(QtCore.QRect(10, 10, 230, 240))
        self.lwgpsseis.setSelectionMode(QtWidgets.QAbstractItemView.ExtendedSelection)
        #
        self.lblpsshot = QtWidgets.QLabel(PlotVis2DPsSeisShot)
        self.lblpsshot.setObjectName("lblpsshot")
        self.lblpsshot.setGeometry(QtCore.QRect(260, 50, 150, 30))
        self.slbpsshot = QtWidgets.QScrollBar(PlotVis2DPsSeisShot)
        self.slbpsshot.setObjectName("slbpsshot")
        self.slbpsshot.setOrientation(QtCore.Qt.Horizontal)
        self.slbpsshot.setGeometry(QtCore.QRect(260, 80, 170, 30))
        self.ldtpsshot = QtWidgets.QLineEdit(PlotVis2DPsSeisShot)
        self.ldtpsshot.setObjectName("ldtpsshot")
        self.ldtpsshot.setGeometry(QtCore.QRect(440, 80, 50, 30))
        self.ldtpsshot.setAlignment(QtCore.Qt.AlignCenter)
        #
        self.lblcmap = QtWidgets.QLabel(PlotVis2DPsSeisShot)
        self.lblcmap.setObjectName("lblcmap")
        self.lblcmap.setGeometry(QtCore.QRect(260, 220, 60, 30))
        self.cbbcmap = QtWidgets.QComboBox(PlotVis2DPsSeisShot)
        self.cbbcmap.setObjectName("cbbcmap")
        self.cbbcmap.setGeometry(QtCore.QRect(320, 220, 120, 30))
        self.cbxflip = QtWidgets.QCheckBox(PlotVis2DPsSeisShot)
        self.cbxflip.setObjectName("cbxflip")
        self.cbxflip.setGeometry(QtCore.QRect(450, 220, 40, 30))
        #
        self.lblrange = QtWidgets.QLabel(PlotVis2DPsSeisShot)
        self.lblrange.setObjectName("lblrange")
        self.lblrange.setGeometry(QtCore.QRect(260, 130, 40, 30))
        self.ldtmin = QtWidgets.QLineEdit(PlotVis2DPsSeisShot)
        self.ldtmin.setObjectName("ldtmin")
        self.ldtmin.setGeometry(QtCore.QRect(300, 130, 70, 30))
        self.ldtmin.setAlignment(QtCore.Qt.AlignCenter)
        self.ldtmax = QtWidgets.QLineEdit(PlotVis2DPsSeisShot)
        self.ldtmax.setObjectName("ldtmax")
        self.ldtmax.setGeometry(QtCore.QRect(420, 130, 70, 30))
        self.ldtmax.setAlignment(QtCore.Qt.AlignCenter)
        self.lblrangeto = QtWidgets.QLabel(PlotVis2DPsSeisShot)
        self.lblrangeto.setObjectName("lblrangeto")
        self.lblrangeto.setGeometry(QtCore.QRect(370, 130, 50, 30))
        self.lblrangeto.setAlignment(QtCore.Qt.AlignCenter)
        #
        self.btnconfigplayer = QtWidgets.QPushButton(PlotVis2DPsSeisShot)
        self.btnconfigplayer.setObjectName("btnconfigplayer")
        self.btnconfigplayer.setGeometry(QtCore.QRect(390, 10, 100, 30))
        icon = QtGui.QIcon()
        icon.addPixmap(QtGui.QPixmap(os.path.join(self.iconpath, "icons/video.png")),
                       QtGui.QIcon.Normal,
                       QtGui.QIcon.Off)
        self.btnconfigplayer.setIcon(icon)
        #
        self.btnplotshot = QtWidgets.QPushButton(PlotVis2DPsSeisShot)
        self.btnplotshot.setObjectName("btnplotshot")
        self.btnplotshot.setGeometry(QtCore.QRect(170, 270, 160, 30))
        icon = QtGui.QIcon()
        icon.addPixmap(QtGui.QPixmap(os.path.join(self.iconpath, "icons/gather.png")),
                       QtGui.QIcon.Normal,
                       QtGui.QIcon.Off)
        self.btnplotshot.setIcon(icon)
        #
        self.msgbox = QtWidgets.QMessageBox(PlotVis2DPsSeisShot)
        self.msgbox.setObjectName("msgbox")
        _center_x = PlotVis2DPsSeisShot.geometry().center().x()
        _center_y = PlotVis2DPsSeisShot.geometry().center().y()
        self.msgbox.setGeometry(QtCore.QRect(_center_x - 150, _center_y - 50, 300, 100))
        #
        self.retranslateGUI(PlotVis2DPsSeisShot)
        QtCore.QMetaObject.connectSlotsByName(PlotVis2DPsSeisShot)



    def retranslateGUI(self, PlotVis2DPsSeisShot):
        self.dialog = PlotVis2DPsSeisShot
        #
        _translate = QtCore.QCoreApplication.translate
        PlotVis2DPsSeisShot.setWindowTitle(_translate("PlotVis2DPsSeisShot", "2D Window: Pre-stack Gather"))
        if self.checkPsSeisData():
            _firstpsseis = None
            for i in sorted(self.psseisdata.keys()):
                item = QtWidgets.QListWidgetItem(self.lwgpsseis)
                item.setText(_translate("PlotVis2DPsSeisShot", i))
                self.lwgpsseis.addItem(item)
                if _firstpsseis is None:
                    _firstpsseis = item
            self.lwgpsseis.setCurrentItem(_firstpsseis)
        self.lwgpsseis.itemSelectionChanged.connect(self.changeLwgPsseis)
        #
        self.lblpsshot.setText(_translate("PlotVis2DPsSeisShot", "@ gather:"))
        self.ldtpsshot.setEnabled(False)
        if (self.lwgpsseis.currentItem() is not None) \
                and (self.lwgpsseis.currentItem().text() in self.psseisdata.keys()):
            _shotlist = list(sorted(self.psseisdata[_firstpsseis.text()].keys()))
            _slicemin = 0
            _slicemax = len(_shotlist) - 1
        else:
            _shotlist = []
            _slicemin = 0
            _slicemax = 0
        self.slbpsshot.setMinimum(_slicemin)
        self.slbpsshot.setMaximum(_slicemax)
        self.slbpsshot.setValue(_slicemin)
        if len(_shotlist) > _slicemin:
            self.ldtpsshot.setText(_translate("PlotVis2DPsSeisShot", _shotlist[_slicemin]))
        else:
            self.ldtpsshot.setText(_translate("PlotVis2DPsSeisShot", ''))
        self.slbpsshot.valueChanged.connect(self.changeSlbPsshot)
        #
        self.lblcmap.setText(_translate("Plot2DXlSlice", "Color map:"))
        self.cbbcmap.addItems(vis_cmap.ColorMapList)
        for _i in range(len(vis_cmap.ColorMapList)):
            self.cbbcmap.setItemIcon(_i, QtGui.QIcon(
                QtGui.QPixmap(os.path.join(self.iconpath, "icons/cmap_" + vis_cmap.ColorMapList[_i] + ".png")).scaled(80, 30)))
        self.cbbcmap.setCurrentIndex(list.index(vis_cmap.ColorMapList, self.plotstyle['Colormap']))
        #
        self.cbxflip.setText(_translate("PlotVis2DPsSeisShot", ""))
        self.cbxflip.setIcon(QtGui.QIcon(
            QtGui.QPixmap(os.path.join(self.iconpath, "icons/flip.png")).scaled(80, 80)))
        #
        self.lblrange.setText(_translate("PlotVis2DPsSeisShot", "Range:"))
        self.lblrangeto.setText(_translate("PlotVis2DPsSeisShot", "~~~"))
        if (self.lwgpsseis.currentItem() is not None) \
                and (self.lwgpsseis.currentItem().text() in self.psseisdata.keys()):
            _min, _max = self.getShotRange(self.lwgpsseis.currentItem().text())
            self.ldtmin.setText(_translate("PlotVis2DPsSeisShot", str(_min)))
            self.ldtmax.setText(_translate("PlotVis2DPsSeisShot", str(_max)))
        #
        self.btnconfigplayer.setText(_translate("PlotVis2DPsSeisShot", "Gather Player"))
        self.btnconfigplayer.clicked.connect(self.clickBtnConfigPlayer)
        #
        self.btnplotshot.setText(_translate("PlotVis2DPsSeisShot", "Pre-stack Gather Viewer"))
        self.btnplotshot.setDefault(True)
        self.btnplotshot.clicked.connect(self.clickBtnPlotShot)


    def changeLwgPsseis(self):
        if len(self.lwgpsseis.selectedItems()) > 0:
            if (self.lwgpsseis.currentItem() is not None) \
                and (self.lwgpsseis.currentItem().text() in self.psseisdata.keys()):
                _shotlist = list(sorted(self.psseisdata[self.lwgpsseis.currentItem().text()].keys()))
                self.slbpsshot.setMinimum(0)
                self.slbpsshot.setMaximum(len(_shotlist) - 1)
                self.slbpsshot.setValue(0)
                self.ldtpsshot.setText(_shotlist[0])
                _min, _max = self.getShotRange(self.lwgpsseis.currentItem().text())
                self.ldtmin.setText(str(_min))
                self.ldtmax.setText(str(_max))
        else:
            self.slbpsshot.setMinimum(0)
            self.slbpsshot.setMaximum(0)
            self.ldtpsshot.setText('')
            self.ldtmin.setText('')
            self.ldtmax.setText('')


    def changeSlbPsshot(self):
        _shotlist = list(sorted(self.psseisdata[self.lwgpsseis.currentItem().text()].keys()))
        self.ldtpsshot.setText(_shotlist[self.slbpsshot.value()])


    def clickBtnConfigPlayer(self):
        _config = QtWidgets.QDialog()
        _gui = gui_configplayer()
        _gui.playerconfig = {}
        _gui.playerconfig['Gather'] = self.playerconfig
        _gui.setupGUI(_config)
        _config.exec()
        self.playerconfig = _gui.playerconfig['Gather']
        _config.show()


    def clickBtnPlotShot(self):
        self.refreshMsgBox()
        #
        _psseislist = self.lwgpsseis.selectedItems()
        if len(_psseislist) < 1:
            vis_msg.print("ERROR in PlotVis2DPsSeisShot: No pre-stack selected for plot", type='error')
            QtWidgets.QMessageBox.critical(self.msgbox,
                                           '2D Window: Pre-stack Gather',
                                           'No pre-stack selected for plot')
            return
        #
        _cmap = self.cbbcmap.currentIndex()
        _flip = self.cbxflip.isChecked()
        _min = basic_data.str2float(self.ldtmin.text())
        _max = basic_data.str2float(self.ldtmax.text())
        if _min is False or _max is False:
            vis_msg.print("ERROR in PlotVis2DPsSeisShot: Non-float range specified for plot", type='error')
            QtWidgets.QMessageBox.critical(self.msgbox,
                                           '2D Window: Pre-stack Gather',
                                           'Non-float range specified for plot')
            return
        #
        for i in range(len(_psseislist)):
            print("PlotVis2DPsSeisShot: Plot %d of %d pre-stack: %s" % (i + 1, len(_psseislist), _psseislist[i].text()))
            _data = self.psseisdata[_psseislist[i].text()]
            #
            _shotlist = list(sorted(self.psseisdata[_psseislist[i].text()].keys()))
            if self.slbpsshot.value() >= len(_shotlist):
                _sls = _shotlist[0]
            else:
                _sls = _shotlist[self.slbpsshot.value()]
            psseis_vis.plotPsSeisShotPlayer(_data, initshot=_sls,
                                            titlesurf=' in ' + _psseislist[i].text(),
                                            valuemin=_min,
                                            valuemax=_max,
                                            colormap=vis_cmap.ColorMapList[_cmap],
                                            flipcmap=_flip, colorbaron=True,
                                            interpolation=self.plotstyle['Interpolation'].lower(),
                                            playerconfig=self.playerconfig,
                                            fontstyle=self.fontstyle,
                                            qicon=QtGui.QIcon(os.path.join(self.iconpath, "icons/logo.png"))
                                            )
        return


    def getShotRange(self, f):
        _min = -1
        _max = 1
        if f in self.psseisdata.keys() and psseis_ays.checkPsSeis(self.psseisdata[f]):
            for k in self.psseisdata[f].keys():
                _vmin = np.min(self.psseisdata[f][k]['ShotData'])
                _vmax = np.max(self.psseisdata[f][k]['ShotData'])
                if _vmin < _min:
                    _min = _vmin
                if _vmax > _max:
                    _max = _vmax
        return _min, _max


    def refreshMsgBox(self):
        _center_x = self.dialog.geometry().center().x()
        _center_y = self.dialog.geometry().center().y()
        self.msgbox.setGeometry(QtCore.QRect(_center_x - 150, _center_y - 50, 300, 100))


    def checkPsSeisData(self):
        self.refreshMsgBox()
        #
        for f in self.psseisdata.keys():
            if psseis_ays.checkPsSeis(self.psseisdata[f]) is False:
                return False
        return True


if __name__ == "__main__":
    app = QtWidgets.QApplication(sys.argv)
    PlotVis2DPsSeisShot = QtWidgets.QWidget()
    gui = plotvis2dpsseisshot()
    gui.setupGUI(PlotVis2DPsSeisShot)
    PlotVis2DPsSeisShot.show()
    sys.exit(app.exec_())


#############################################################################################
#                                                                                           #
# Author:   GeoPy Team                                                                      #
# Email:    geopy.info@gmail.com                                                            #
# Date:     September 2018                                                                  #
#                                                                                           #
#############################################################################################

# seismic attribute analysis functions

from PyQt5 import QtCore
import sys, os
import numpy as np
from scipy.signal.signaltools import hilbert
#
sys.path.append(os.path.dirname(__file__)[:-4][:-4][:-13])
from cognitivegeo.src.vis.messager import messager as vis_msg


def calcPowerSpectrum(seis3dmat, zstep):
    """
    Calculate the power spectrum of a seismic cube from Seis3DMat

    Args:
        seis3dmat:  seismic data in 3D matrix [Z/XL/IL]
        zstep:      z sampling rate in ms

    Return:
        spec:       2D matrix of two columns, [frequency, spectrum]
    """

    if np.ndim(seis3dmat) != 3:
        vis_msg.print('ERROR in calcPowerSpectrum: 3D seismic matrix expected', type='error')
        sys.exit()
    #
    znum = np.shape(seis3dmat)[0]
    #
    spec = np.zeros([int((znum+1)/2), 2])
    #
    freq = np.fft.fft(seis3dmat, axis=0)
    freq = np.abs(freq)
    freq = np.mean(freq, axis=1)
    freq = np.mean(freq, axis=1)
    #
    spec[:, 0] = np.linspace(0, 500.0/np.abs(zstep), int((znum+1)/2))
    spec[:, 1] = freq[0:int((znum+1)/2)]
    #
    return spec


def calcCumulativeSum(seis3dmat):
    """
    Calculate cusum attribute

    Args:
        seis3dmat: seismic data in 3D matrix [Z/XL/IL]

    Return:
        cusum attribute as 3D matrix
    """

    if np.ndim(seis3dmat) != 3:
        vis_msg.print('ERROR in calcCumulativeSum: 3D seismic matrix expected', type='error')
        sys.exit()
    #
    return np.cumsum(seis3dmat, axis=0)

def calcFirstDerivative(seis3dmat):
    """
    Calculate first derivative attribute

    Args:
        seis3dmat: seismic data in 3D matrix [Z/XL/IL]

    Return:
        first-derivative attribute as 3D matrix
    """
    if np.ndim(seis3dmat) != 3:
        vis_msg.print('ERROR in calcFirstDerivative: 3D seismic matrix expected', type='error')
        sys.exit()
    #
    attrib = seis3dmat.copy()
    if np.shape(seis3dmat)[0] > 1:
        attrib[1:, :, :] -= seis3dmat[0:-1, :, :]
        # attrib[0, :, :] *= 0
    #
    return attrib

def calcInstanEnvelop(seis3dmat):
    """
    Calculate instantaneous envelop attribute

    Args:
        seis3dmat: seismic data in 3D matrix [Z/XL/IL]

    Return:
        Instantaneous envelop attribute as 3D matrix
    """

    if np.ndim(seis3dmat) != 3:
        vis_msg.print('ERROR in calcInstanEnvelop: 3D seismic matrix expected', type='error')
        sys.exit()
    #
    attrib = np.abs(hilbert(seis3dmat, axis=0))
    #
    return attrib

def calcInstanQuadrature(seis3dmat):
    """
    Calculate instantaneous quadrature attribute

    Args:
        seis3dmat: seismic data in 3D matrix [Z/XL/IL]

    Return:
        Instantaneous quadrature attribute as 3D matrix
    """

    if np.ndim(seis3dmat) != 3:
        vis_msg.print('ERROR in calcInstanEnvelop: 3D seismic matrix expected', type='error')
        sys.exit()
    #
    attrib = np.imag(hilbert(seis3dmat, axis=0))
    #
    return attrib

def calcInstanPhase(seis3dmat):
    """
    Calculate instantaneous phase attribute

    Args:
        seis3dmat: seismic data in 3D matrix [Z/XL/IL]

    Return:
        Instantaneous phase attribute as 3D matrix
    """

    if np.ndim(seis3dmat) != 3:
        vis_msg.print('ERROR in calcInstanEnvelop: 3D seismic matrix expected', type='error')
        sys.exit()
    #
    attrib = np.angle(hilbert(seis3dmat, axis=0))
    attrib = attrib * 180.0 / np.pi
    #
    return attrib

def calcInstanFrequency(seis3dmat):
    """
    Calculate instantaneous frequency attribute

    Args:
        seis3dmat: seismic data in 3D matrix [Z/XL/IL]

    Return:
        Instantaneous frequency attribute as 3D matrix
    """

    if np.ndim(seis3dmat) != 3:
        vis_msg.print('ERROR in calcInstanEnvelop: 3D seismic matrix expected', type='error')
        sys.exit()
    #
    instphase = np.unwrap(np.angle(hilbert(seis3dmat, axis=0)), axis=0) * 0.5 / np.pi
    #
    attrib = np.zeros(np.shape(seis3dmat))
    attrib[1:-1, :, :] = 0.5 * (instphase[2:, :, :] - instphase[0:-2, :, :])
    #
    return attrib

def calcInstanCosPhase(seis3dmat):
    """
    Calculate instantaneous cosine of phase attribute

    Args:
        seis3dmat: seismic data in 3D matrix [Z/XL/IL]

    Return:
        Cosine of phase attribute as 3D matrix
    """

    if np.ndim(seis3dmat) != 3:
        vis_msg.print('ERROR in calcInstanEnvelop: 3D seismic matrix expected', type='error')
        sys.exit()
    #
    attrib = np.angle(hilbert(seis3dmat, axis=0))
    attrib = np.cos(attrib)
    #
    return attrib


class attribute:
    # pack all functions as a class
    #
    calcPowerSpectrum = calcPowerSpectrum
    #
    calcCumulativeSum = calcCumulativeSum
    calcFirstDerivative = calcFirstDerivative
    #
    calcInstanEnvelop = calcInstanEnvelop
    calcInstanQuadrature = calcInstanQuadrature
    calcInstanPhase = calcInstanPhase
    calcInstanFrequency = calcInstanFrequency
    calcInstanCosPhase = calcInstanCosPhase


#############################################################################################
#                                                                                           #
# Author:   GeoPy Team                                                                      #
# Email:    geopy.info@gmail.com                                                            #
# Date:     January 2019                                                                    #
#                                                                                           #
#############################################################################################

# basic functions for markers

import sys, os
#
sys.path.append(os.path.dirname(__file__)[:-4][:-4][:-13])
from cognitivegeo.src.vis.color import color as color


__all__ = ['marker']

# List of all available marker properties, including Style and Size
MarkerStyleList = ['*', '+', 'o', 'v', '^', '<', '<',
                   'x', 'X', '.', 'None']
MarkerSizeList = [i for i in range(1, 20)]


class marker:
    # Pack all functions as a class
    #
    MarkerStyleList = MarkerStyleList
    MarkerSizeList = MarkerSizeList
    MarkerColorList = color.ColorList


#############################################################################################
#                                                                                           #
# Author:   GeoPy Team                                                                      #
# Email:    geopy.info@gmail.com                                                            #
# Date:     January 2019                                                                    #
#                                                                                           #
#############################################################################################

# basic functions for colormap

import numpy as np
import matplotlib.colors as clr


__all__ = ['colormap']

# List of all available colormaps
ColorMapList = ['Seismic', 'Phase', 'Frequency', 'Red-White-Blue', 'Gray',
                'Black-White-Red', 'Black-White-Green', 'Black-White-Blue',
                'White-Red-Black', 'White-Green-Black', 'White-Blue-Black',
                'Black-Red', 'Black-Green', 'Black-Blue']

# List of all available opacity optition
OpacityList = ['0%', '10%', '20%', '30%', '40%', '50%', '60%', '70%', '80%', '90%', '100%',
               '___|\u207B\u207B\u207B', '\u207B\u207B\u207B|___',
               '__|\u207B\u207B|__', '\u207B\u207B|__|\u207B\u207B']


def makeOpacity(opacityname=None):
    if opacityname is None:
        opacityname = '100%'

    # default 100%
    opac = np.ones([2001, 1])

    # 0-100%
    if opacityname == '0%':
        opac = opac * 0.0
    if opacityname == '10%':
        opac = opac * 0.1
    if opacityname == '20%':
        opac = opac * 0.2
    if opacityname == '30%':
        opac = opac * 0.3
    if opacityname == '40%':
        opac = opac * 0.4
    if opacityname == '50%':
        opac = opac * 0.5
    if opacityname == '60%':
        opac = opac * 0.6
    if opacityname == '70%':
        opac = opac * 0.7
    if opacityname == '80%':
        opac = opac * 0.8
    if opacityname == '90%':
        opac = opac * 0.9
    if opacityname == '100%':
        opac = opac * 1.0

    # more opacity
    if opacityname == OpacityList[11]:
        opac[0:1001, :] = opac[0:1001, :] * 0.0
    if opacityname == OpacityList[12]:
        opac[1000:, :] = opac[1000:, :] * 0.0
    if opacityname == OpacityList[13]:
        opac[0:667, :] = opac[0:667, :] * 0.0
        opac[1334:, :] = opac[1334:, :] * 0.0
    if opacityname == OpacityList[14]:
        opac[667:1334, :] = opac[667:1334, :] * 0.0

    #
    return opac


def makeColorMap(cmapname=None, flip=False, opacity='100%'):
    """
    Make common colormap used for data visualization
    Argus:
        cmapname:   name of colormap as listed below:
                        'Seismic', 'Phase', 'Frequency',
                        'Red-White-Blue',
                        'Black-White-Red', 'Black-White-Green', 'Black-White-Blue',
                        'Gray',
                        'White-Red-Black', 'White-Green-Black', 'White-Blue-Black',
                        'Black-Red', 'Black-Green', 'Black-Blue'
                    Default is 'Red-White-Blue'
        flip:       flip colormap or not
    Return:
         colormap
    """

    if cmapname is None:
        cmapname = 'Red-White-Blue'

    colormap = {}

    # alpha
    col_a = makeOpacity(opacity)

    # seismic
    col_loc = np.array([-1.0, -0.33, -0.2, 0.0, 0.2, 0.33, 1.0])
    col_r = np.array([161.0, 0.0, 77.0, 204.0, 97.0, 191.0, 255.0]) / 255.0
    col_g = np.array([255.0, 0.0, 77.0, 204.0, 69.0, 0.0, 255.0]) / 255.0
    col_b = np.array([255.0, 191.0, 77.0, 204.0, 0.0, 0.0, 0.0]) / 255.0
    col_r = np.interp(np.linspace(-1.0, 1.0, 2001), col_loc, col_r)
    col_g = np.interp(np.linspace(-1.0, 1.0, 2001), col_loc, col_g)
    col_b = np.interp(np.linspace(-1.0, 1.0, 2001), col_loc, col_b)
    col_r = np.reshape(col_r, [2001, 1])
    col_g = np.reshape(col_g, [2001, 1])
    col_b = np.reshape(col_b, [2001, 1])
    seismic = np.concatenate((col_r, col_g, col_b), axis=1)
    # Add to colormap dictionary
    colormap['Seismic'] = seismic

    # phase
    col_loc = np.linspace(0.0, 1.0, 11)
    col_r = np.array([255.0, 255.0, 255.0, 255.0, 198.0, 0.0, 0.0, 0.0, 0.0, 161.0, 255.0]) / 255.0
    col_g = np.array([0.0, 0.0, 114.0, 228.0, 255.0, 255.0, 255.0, 228.0, 114.0, 0.0, 0.0]) / 255.0
    col_b = np.array([255.0, 161.0, 0.0, 0.0, 0.0, 0.0, 198.0, 255.0, 255.0, 255.0, 255.0]) / 255.0
    col_r = np.interp(np.linspace(0.0, 1.0, 2001), col_loc, col_r)
    col_g = np.interp(np.linspace(0.0, 1.0, 2001), col_loc, col_g)
    col_b = np.interp(np.linspace(0.0, 1.0, 2001), col_loc, col_b)
    col_r = np.reshape(col_r, [2001, 1])
    col_g = np.reshape(col_g, [2001, 1])
    col_b = np.reshape(col_b, [2001, 1])
    phase = np.concatenate((col_r, col_g, col_b), axis=1)
    # Add to colormap dictionary
    colormap['Phase'] = phase

    # frequency
    col_loc = np.linspace(0.0, 1.0, 11)
    col_r = np.array([0.0, 255.0, 255.0, 240.0, 147.0, 0.0, 0.0, 0.0, 0.0, 170.0, 255.0]) / 255.0
    col_g = np.array([0.0, 0.0, 190.0, 255.0, 255.0, 255.0, 255.0, 208.0, 85.0, 0.0, 0.0]) / 255.0
    col_b = np.array([0.0, 0.0, 0.0, 0.0, 0.0, 120.0, 225.0, 255.0, 255.0, 255.0, 255.0]) / 255.0
    col_r = np.interp(np.linspace(0.0, 1.0, 2001), col_loc, col_r)
    col_g = np.interp(np.linspace(0.0, 1.0, 2001), col_loc, col_g)
    col_b = np.interp(np.linspace(0.0, 1.0, 2001), col_loc, col_b)
    col_r = np.reshape(col_r, [2001, 1])
    col_g = np.reshape(col_g, [2001, 1])
    col_b = np.reshape(col_b, [2001, 1])
    frequency = np.concatenate((col_r, col_g, col_b), axis=1)
    # Add to colormap dictionary
    colormap['Frequency'] = frequency

    # red_white_blue
    col_r = np.concatenate((np.ones([1001]), np.linspace(0.999, 0.0, 1000)))
    col_r = np.reshape(col_r, [2001, 1])
    col_g = np.concatenate((np.linspace(0.0, 1.0, 1001), np.linspace(0.999, 0.0, 1000)))
    col_g = np.reshape(col_g, [2001, 1])
    col_b = np.concatenate((np.linspace(0.0, 0.999, 1000), np.ones([1001])))
    col_b = np.reshape(col_b, [2001, 1])
    red_white_blue = np.concatenate((col_r, col_g, col_b), axis=1)
    # Add to colormap dictionary
    colormap['Red-White-Blue'] = red_white_blue

    # black_white_red/green/blue
    col_1 = np.concatenate((np.linspace(0.0, 0.999, 1000), np.ones([1001])))
    col_1 = np.reshape(col_1, [2001, 1])
    col_2 = np.concatenate((np.linspace(0.0, 1.0, 1001), np.linspace(0.999, 0.0, 1000)))
    col_2 = np.reshape(col_2, [2001, 1])
    black_white_red = np.concatenate((col_1, col_2, col_2), axis=1)
    black_white_green = np.concatenate((col_2, col_1, col_2), axis=1)
    black_white_blue = np.concatenate((col_2, col_2, col_1), axis=1)
    # Add to colormap dictionary
    colormap['Black-White-Red'] = black_white_red
    colormap['Black-White-Green'] = black_white_green
    colormap['Black-White-Blue'] = black_white_blue

    # white_gray_black
    col = np.linspace(1.0, 0.0, 2001)
    col = np.reshape(col, [2001, 1])
    white_gray_black = np.concatenate((col, col, col), axis=1)
    # Add to colormap dictionary
    colormap['Gray'] = white_gray_black

    # white_red/green/blue_black
    col_1 = np.concatenate((np.ones([1001]), np.linspace(0.999, 0.0, 1000)))
    col_1 = np.reshape(col_1, [2001, 1])
    col_2 = np.concatenate((np.linspace(1.0, 0.0, 1001), np.zeros([1000])))
    col_2 = np.reshape(col_2, [2001, 1])
    white_red_black = np.concatenate((col_1, col_2, col_2), axis=1)
    white_green_black = np.concatenate((col_2, col_1, col_2), axis=1)
    white_blue_black = np.concatenate((col_2, col_2, col_1), axis=1)
    # Add to colormap dictionary
    colormap['White-Red-Black'] = white_red_black
    colormap['White-Green-Black'] = white_green_black
    colormap['White-Blue-Black'] = white_blue_black

    # black_red/green/blue
    col_1 = np.linspace(0.0, 1.0, 2001)
    col_1 = np.reshape(col_1, [2001, 1])
    col_2 = np.zeros([2001])
    col_2 = np.reshape(col_2, [2001, 1])
    black_red = np.concatenate((col_1, col_2, col_2), axis=1)
    black_green = np.concatenate((col_2, col_1, col_2), axis=1)
    black_blue = np.concatenate((col_2, col_2, col_1), axis=1)
    # Add to colormap dictionary
    colormap['Black-Red'] = black_red
    colormap['Black-Green'] = black_green
    colormap['Black-Blue'] = black_blue

    cmapdata = colormap['Red-White-Blue']
    if cmapname in colormap:
        cmapdata = colormap[cmapname]
    if flip:
        cmapdata = np.flipud(cmapdata)

    cmapdata = np.concatenate((cmapdata, col_a), axis=1)

    return clr.ListedColormap(cmapdata)


class colormap:
    # Pack all functions as a class
    #
    ColorMapList = ColorMapList
    OpacityList = OpacityList
    #
    makeColorMap = makeColorMap


#############################################################################################
#                                                                                           #
# Author:   GeoPy Team                                                                      #
# Email:    geopy.info@gmail.com                                                            #
# Date:     January 2019                                                                    #
#                                                                                           #
#############################################################################################

# basic functions for crossplot

import sys, os
import numpy as np
import matplotlib.pyplot as plt
#
sys.path.append(os.path.dirname(__file__)[:-9][:-4][:-13])
from cognitivegeo.src.basic.matdict import matdict as basic_mdt
from cognitivegeo.src.vis.font import font as vis_font
from cognitivegeo.src.vis.messager import messager as vis_msg

__all__ = ['visualization']


def crossplot2D(pointdict,
                colorlist=[], linestylelist=[],
                linewidthlist=[],
                markerstylelist=[],
                markersizelist=[],
                xfeature='x', xlabel='x-label',
                yfeature='y', ylabel='y-label',
                xlim=[-10, 10], ylim=[-10, 10],
                fontstyle=None,
                title=None,
                legendon=True,
                qicon=None, qtitle='2D Window - PointSet Cross-plot'):
    """
    2D corss plot

    Args:
        pointdict:          Point dictionary, with each key representing a point group
        colorlist:          List of the colors for each point group
        linestylelist:      List of the line style for each point group
        linewidthlist:      List of the line width for each point group
        markerstylelist:    List of the marker style for each point group
        markersizelist:     List of the marker size for each point group
        xfeature:           x-axis feature. It must be within the keys in each point group
        xlabel:             x-axis label to display
        yfeature:           y-axis feature. It must be within the keys in each point group
        ylabel:             y-axis label to display
        xlim:               x-axi limit to display
        ylim:               y-axis limit to display
        fontstyle:          font style to display
        title:              title to display
        legendon:           Legend to be displayed or not. Default is True
        qicon:              QIcon to be used for the window
        qtitle:             QTitle to be used for the window

    Return:
        None
    """


    # if len(colorlist) < len(pointdict):
    #     print('WARNING in crossplot2D: No color coding')
    #     colorlist = []
    # if len(markerlist) < len(pointdict):
    #     print('WARNING in crossplot2D: No marker coding')
    #     markerlist = []

    vis_font.updatePltFont(fontstyle)
    #
    fig = plt.figure(facecolor='white')
    for idx, name in enumerate(pointdict):
        color = 'Black'
        if idx < len(colorlist):
            color = colorlist[idx]
        linestyle = 'Solid'
        if idx < len(linestylelist):
            linestyle = linestylelist[idx]
        linewidth = 12
        if idx < len(linewidthlist):
            linewidth = linewidthlist[idx]
        markerstyle = '.'
        if idx < len(markerstylelist):
            markerstyle = markerstylelist[idx]
        markersize = '.'
        if idx < len(markersizelist):
            markersize = markersizelist[idx]
        #
        pointdata = pointdict[name]
        if xfeature not in pointdata.keys():
            vis_msg.print('ERROR in crossplot2D: %s not found in %s' %(xfeature, name), type='error')
            sys.exit()
        if yfeature not in pointdata.keys():
            vis_msg.print('ERROR in crossplot2D: %s not found in %s' %(yfeature, name), type='error')
            sys.exit()
        #
        pointnum = basic_mdt.maxDictConstantRow(pointdata)
        x = np.mean(np.reshape(pointdata[xfeature], [pointnum, -1]),
                    axis=1)
        y = np.mean(np.reshape(pointdata[yfeature], [pointnum, -1]),
                    axis=1)
        plt.plot(x, y, linestyle=linestyle, linewidth=linewidth,
                 color=color, marker=markerstyle, markersize=markersize, label=name)
    #
    if legendon:
        plt.legend()
    #
    plt.xlabel(xlabel)
    plt.xlim(xlim)
    plt.ylabel(ylabel)
    plt.ylim(ylim)
    if title is None:
        title = xfeature + ' vs ' + yfeature
    plt.title(title)

    if qicon is not None:
        fig.canvas.set_window_title(qtitle)
        #
        # Commented by HD on June 7, 2018 to avoid crash
        # plt.get_current_fig_manager().window.setWindowIcon(qicon)

    plt.show()


class visualization:
    # Pack all functions as a class
    #
    crossplot2D = crossplot2D


#############################################################################################
#                                                                                           #
# Author:   GeoPy Team                                                                      #
# Email:    geopy.info@gmail.com                                                            #
# Date:     January 2019                                                                    #
#                                                                                           #
#############################################################################################

# pre-stack seismic data processing functions

from PyQt5 import QtCore
import sys, os
import numpy as np
#
sys.path.append(os.path.dirname(__file__)[:-4][:-4][:-13])
from cognitivegeo.src.vis.messager import messager as vis_msg


__all__ = ['analysis']

# Shot information is packed as a dictionary contains the following keys,
#   'ZNum' ----- number of z samples
#   'ZStart' ----- first z
#   'ZStep' ----- z step
#   'ZEnd' ------ last z
#   'ZRange' ----- all z as a 1D array
#   'XLNum' ----- number of xlines
#   'XLStart' ----- first xlines
#   'XLStep' ----- xline step
#   'XLEnd' ------ last xline
#   'XLRange' ----- all xlines as a 1D array
#   'ILNum' ----- number of inlines
#   'ILStart' ----- first inlines
#   'ILStep' ----- inline step
#   'ILEnd' ------ last inline
#   'ILRange' ----- all inlines as a 1D array
#   'TraeFlag' ----- 2D array of the missing-trace flag. 1 for missing trace


def checkPsSeis(psseis):
    """
    Check if a pre-stack seismic dictionary good to use

    Args:
        psseis: A pre-stack seismic dictionary, with each key representing a shot dictionary.
                A shot dictionary contains the following keys,
                    'ShotData' ----- Shot data as a 3D array [Z/XL/IL]
                    'ShotInfo' ----- Shot information as a dictionary defined above
    Return:
    """

    if len(psseis.keys()) < 1:
        return False
    #
    for shot in psseis.keys():
        if 'ShotData' not in psseis[shot].keys() or 'ShotInfo' not in psseis[shot].keys():
            return False
        if 'ZNum' not in psseis[shot]['ShotInfo'].keys():
            return False
        if 'ZStart' not in psseis[shot]['ShotInfo'].keys():
            return False
        if 'ZEnd' not in psseis[shot]['ShotInfo'].keys():
            return False
        if 'ZStep' not in psseis[shot]['ShotInfo'].keys():
            return False
        if 'ZRange' not in psseis[shot]['ShotInfo'].keys():
            return False
        if 'XLNum' not in psseis[shot]['ShotInfo'].keys():
            return False
        if 'XLStart' not in psseis[shot]['ShotInfo'].keys():
            return False
        if 'XLEnd' not in psseis[shot]['ShotInfo'].keys():
            return False
        if 'XLStep' not in psseis[shot]['ShotInfo'].keys():
            return False
        if 'XLRange' not in psseis[shot]['ShotInfo'].keys():
            return False
        if 'ILNum' not in psseis[shot]['ShotInfo'].keys():
            return False
        if 'ILStart' not in psseis[shot]['ShotInfo'].keys():
            return False
        if 'ILEnd' not in psseis[shot]['ShotInfo'].keys():
            return False
        if 'ILStep' not in psseis[shot]['ShotInfo'].keys():
            return False
        if 'ILRange' not in psseis[shot]['ShotInfo'].keys():
            return False
        if 'TraceFlag' not in psseis[shot]['ShotInfo'].keys():
            return False
    #
    return True


def createShotInfoFromShotData(shotdata, zstart=0, zstep=-1, xlstart=0, xlstep=1, inlstart=0, inlstep=1):
    """
    Create a ShotInfo dictionary from shot data, with given parameters.

    Args:
        shotdata:   3D array of the shot data [Z/XL/IL]
        zstart:     First z. Default is 0
        zstep:      Step along z (negative). Default is -1
        xlstart:    First xline. Default is 0
        xlstep:     Step of xline. Default is 1
        inlstart:   First inline. Default is 0
        inlstep:    Step of inline. Default is 1

    Return:
         A shot information dictionary as defined above
    """

    info = makeShotInfo()
    if np.ndim(shotdata) < 3:
        vis_msg.print('ERROR in createShotInfo: shot data in 3D matrix', type='error')
        sys.exit()
    #
    info['ZNum'], info['XLNum'], info['ILNum'] = np.shape(shotdata)
    info['ZStart'] = zstart
    info['ZStep'] = zstep
    info['ZEnd'] = zstart + (info['ZNum'] - 1) * zstep
    info['ZRange'] = np.linspace(info['ZStart'], info['ZEnd'], info['ZNum'])
    info['XLStart'] = xlstart
    info['XLStep'] = xlstep
    info['XLEnd'] = xlstart + (info['XLNum'] - 1) * xlstep
    info['XLRange'] = np.linspace(info['XLStart'], info['XLEnd'], info['XLNum'])
    info['ILStart'] = inlstart
    info['ILStep'] = inlstep
    info['ILEnd'] = inlstart + (info['ILNum'] - 1) * inlstep
    info['ILRange'] = np.linspace(info['ILStart'], info['ILEnd'], info['ILNum'])
    info['TraceFlag'] = np.zeros([info['XLNum'], info['ILNum']])
    # check trace flag
    for i in range(info['ILNum']):
        for j in range(info['XLNum']):
            if np.min(shotdata[:, j, i]) >= np.max(shotdata[:, j, i]):
                info['TraceFlag'][j, i] = 1
    #
    return info


def makeShotInfo(inlstart=0, inlstep=1, inlnum=1,
                 xlstart=0, xlstep=1, xlnum=1,
                 zstart=0, zstep=-1, znum=1):
    """
    Make a new shot information dictionary

    Args:
        inlstart:   first inline. Default is 0
        inlstep:    inline step. Default is 1
        inlnum:     inline number. Default is 1
        xlstart:    first xline. Default is 0
        xlstep:     xline step. Default is 1
        xlnum:      xline number. Default is 1
        zstart:     first z. Default is 0
        zstep:      z step. Default is -1
        znum:       z number. Default is 1

    Return:
        Seismic inforamtion dictionary as defined above
    """

    shotinfo = {}
    shotinfo['ILStart'] = inlstart
    shotinfo['ILEnd'] = inlstart + (inlnum - 1) * inlstep
    shotinfo['ILStep'] = inlstep
    shotinfo['ILNum'] = inlnum
    shotinfo['ILRange'] = np.linspace(0, inlnum - 1, 1) * inlstep + inlstart
    shotinfo['XLStart'] = xlstart
    shotinfo['XLEnd'] = xlstart + (xlnum - 1) * xlstep
    shotinfo['XLStep'] = xlstep
    shotinfo['XLNum'] = xlnum
    shotinfo['XLRange'] = np.linspace(0, inlnum - 1, 1) * xlstep + xlstart
    shotinfo['ZStart'] = zstart
    shotinfo['ZEnd'] = zstart + (znum - 1) * zstep
    shotinfo['ZStep'] = zstep
    shotinfo['ZNum'] = znum
    shotinfo['ZRange'] = np.linspace(0, inlnum - 1, 1) * zstep + zstart
    shotinfo['TraceFlag'] = np.zeros([shotinfo['XLNum'], shotinfo['ILNum']])

    return shotinfo


class analysis:
    checkPsSeis = checkPsSeis
    #
    createShotInfoFromShotData = createShotInfoFromShotData
    #
    makeShotInfo = makeShotInfo